from numpy import dtype
from typing import Mapping, Literal
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np
from copy import deepcopy

from . import gl
from ..color import Color
from ..util import logger

__all__ = (
    "set_viewport",
    "set_depth_range",
    "set_front_face",  # noqa
    "set_cull_face",
    "set_line_width",
    "set_polygon_offset",  # noqa
    "clear",
    "set_clear_color",
    "set_clear_depth",
    "set_clear_stencil",  # noqa
    "set_blend_func",
    "set_blend_color",
    "set_blend_equation",  # noqa
    "set_scissor",
    "set_stencil_func",
    "set_stencil_mask",  # noqa
    "set_stencil_op",
    "set_depth_func",
    "set_depth_mask",  # noqa
    "set_color_mask",
    "set_sample_coverage",  # noqa
    "get_state_presets",
    "set_state",
    "finish",
    "flush",  # noqa
    "read_pixels",
    "set_hint",  # noqa
    "get_gl_configuration",
    "_check_valid",
    "GL_PRESETS",
    "GlooFunctions",
    "global_gloo_functions",
)

_setters: list = ...

# NOTE: If these are updated to have things beyond glEnable/glBlendFunc
# calls, set_state will need to be updated to deal with it.
#: Some OpenGL state presets for common use cases: 'opaque', 'translucent',
#: 'additive'.
#:
#: To be used in :func:`.set_state`.
GL_PRESETS: dict = ...

def get_current_canvas(): ...

# Helpers that are needed for efficient wrapping

def _check_valid(key, val, valid): ...
def _to_args(x): ...
def _check_conversion(key, valid_dict): ...

class BaseGlooFunctions(object):

    ##########################################################################
    # PRIMITIVE/VERTEX

    #
    # Viewport, DepthRangef, CullFace, FrontFace, LineWidth, PolygonOffset
    #

    def set_viewport(self, *args): ...
    def set_depth_range(self, near: float = 0.0, far: float = 1.0): ...
    def set_front_face(self, mode: str = "ccw"): ...
    def set_cull_face(self, mode: str = "back"): ...
    def set_line_width(self, width: float = 1.0): ...
    def set_polygon_offset(self, factor: float = 0.0, units: float = 0.0): ...

    ##########################################################################
    # FRAGMENT/SCREEN

    #
    # glClear, glClearColor, glClearDepthf, glClearStencil
    #

    def clear(self, color=True, depth: bool | float = True, stencil: bool | int = True): ...
    def set_clear_color(self, color="black", alpha: None | float = None): ...
    def set_clear_depth(self, depth: float = 1.0): ...
    def set_clear_stencil(self, index: int = 0): ...

    # glBlendFunc(Separate), glBlendColor, glBlendEquation(Separate)

    def set_blend_func(
        self,
        srgb: str = "one",
        drgb: str = "zero",
        salpha: str | None = None,
        dalpha: str | None = None,
    ): ...
    def set_blend_color(self, color): ...
    def set_blend_equation(self, mode_rgb: str, mode_alpha: str | None = None): ...

    # glScissor, glStencilFunc(Separate), glStencilMask(Separate),
    # glStencilOp(Separate),

    def set_scissor(self, x: int, y: int, w: int, h: int): ...
    def set_stencil_func(
        self,
        func: str = "always",
        ref: int = 0,
        mask: int = 8,
        face: str = "front_and_back",
    ): ...
    def set_stencil_mask(self, mask: int = 8, face: str = "front_and_back"): ...
    def set_stencil_op(
        self,
        sfail: str = "keep",
        dpfail: str = "keep",
        dppass: str = "keep",
        face: str = "front_and_back",
    ): ...

    # glDepthFunc, glDepthMask, glColorMask, glSampleCoverage

    def set_depth_func(self, func: str = "less"): ...
    def set_depth_mask(self, flag: bool): ...
    def set_color_mask(self, red: bool, green: bool, blue: bool, alpha: bool): ...
    def set_sample_coverage(self, value: float = 1.0, invert: bool = False): ...

    ##########################################################################
    # STATE

    #
    # glEnable/Disable
    #

    def get_state_presets(self) -> Mapping: ...
    def set_state(self, preset: Literal["opaque", "translucent", "additive"] | None = None, **kwargs): ...

    #
    # glFinish, glFlush, glReadPixels, glHint
    #

    def finish(self): ...
    def flush(self): ...
    def set_hint(self, target: str, mode: str): ...

class GlooFunctions(BaseGlooFunctions):
    @property
    def glir(self): ...

# Create global functions object and inject names here

# GlooFunctions without queue: use queue of canvas that is current at call-time
global_gloo_functions = ...

# Functions that do not use the glir queue

def read_pixels(
    viewport: ArrayLike | None = None,
    alpha: bool = True,
    mode: str = "color",
    out_type: str | np.dtype = "unsigned_byte",
) -> ArrayLike: ...
def get_gl_configuration() -> Mapping: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import os
import sys
import ctypes.util

from . import _copy_gl_functions
from ._constants import *  # noqa
from ...util import logger

# Ctypes stuff

# Load the OpenGL library. We more or less follow the same approach
# as PyOpenGL does internally

_have_get_proc_address: bool = ...
_lib = ...

def _have_context(): ...
def _get_gl_version(_lib): ...
def _get_gl_func(name, restype, argtypes): ...

# Inject

from . import _gl2  # noqa



from typing import Mapping
from ...classes.graph import Graph
from ...utils.decorators import not_implemented_for

__all__ = ["degree_centrality", "in_degree_centrality", "out_degree_centrality"]

def degree_centrality(G: Graph) -> Mapping: ...
def in_degree_centrality(G: Graph) -> Mapping: ...
def out_degree_centrality(G: Graph) -> Mapping: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.io.matlab.mio_utils, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__pyx_capi__: dict
__test__: dict
def chars_to_strings() -> typing.Any:
    " Convert final axis of char array to strings\n\n    Parameters\n    ----------\n    in_arr : array\n       dtype of 'U1'\n\n    Returns\n    -------\n    str_arr : array\n       dtype of 'UN' where N is the length of the last dimension of\n       ``arr``\n    "
    ...

def squeeze_element() -> typing.Any:
    ' Return squeezed element\n\n    The returned object may not be an ndarray - for example if we do\n    ``arr.item`` to return a ``mat_struct`` object from a struct array '
    ...

def __getattr__(name) -> typing.Any:
    ...




from .utils import (
    assert_nodes_equal as assert_nodes_equal,
    assert_edges_equal as assert_edges_equal,
    assert_graphs_equal as assert_graphs_equal,
    almost_equal as almost_equal,
)
from .test import run as run



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._minpack, version:  1.10 
import typing
import builtins as _mod_builtins
import minpack as _mod_minpack

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__version__: str
def _chkder(m, n, x, fvec, fjac, ldfjac, xp, fvecp, mode, err) -> typing.Any:
    '_chkder(m,n,x,fvec,fjac,ldfjac,xp,fvecp,mode,err)'
    ...

def _hybrd(fun, x0, args, full_output, xtol, maxfev, ml, mu, epsfcn, factor, diag) -> typing.Any:
    '[x,infodict,info] = _hybrd(fun, x0, args, full_output, xtol, maxfev, ml, mu, epsfcn, factor, diag)'
    ...

def _hybrj(fun, Dfun, x0, args, full_output, col_deriv, xtol, maxfev, factor, diag) -> typing.Any:
    '[x,infodict,info] = _hybrj(fun, Dfun, x0, args, full_output, col_deriv, xtol, maxfev, factor, diag)'
    ...

def _lmder(fun, Dfun, x0, args, full_output, col_deriv, ftol, xtol, gtol, maxfev, factor, diag) -> typing.Any:
    '[x,infodict,info] = _lmder(fun, Dfun, x0, args, full_output, col_deriv, ftol, xtol, gtol, maxfev, factor, diag)'
    ...

def _lmdif(fun, x0, args, full_output, ftol, xtol, gtol, maxfev, epsfcn, factor, diag) -> typing.Any:
    '[x,infodict,info] = _lmdif(fun, x0, args, full_output, ftol, xtol, gtol, maxfev, epsfcn, factor, diag)'
    ...

error = _mod_minpack.error
def __getattr__(name) -> typing.Any:
    ...




from collections.abc import Iterable
from vispy.util.svg.color import Color

from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import math

import numpy as np

from .visual import CompoundVisual, updating_property
from .line import LineVisual
from .text import TextVisual

# XXX TODO list (see code, plus):
# 1. Automated tick direction?
# 2. Expand to 3D (only 2D supported currently)
# 3. Input validation
# 4. Property support
# 5. Reactivity to resizing (current tick lengths grow/shrink w/zoom)
# 6. Improve tick label naming (str(x) is not good) and tick selection

class AxisVisual(CompoundVisual):
    def __init__(
        self,
        pos: ArrayLike | None = None,
        domain: tuple = ...,
        tick_direction: ArrayLike = ...,
        scale_type: str = "linear",
        axis_color: tuple = ...,
        tick_color: tuple = ...,
        text_color: Color | str = "w",
        minor_tick_length: float = 5,
        major_tick_length: float = 10,
        tick_width: float = 2,
        tick_label_margin: float = 12,
        tick_font_size: float = 8,
        axis_width: float = 3,
        axis_label: str | None = None,
        axis_label_margin: float = 35,
        axis_font_size: float = 10,
        font_size: float | None = None,
        anchors: Iterable | None = None,
    ): ...
    @property
    def text_color(self): ...
    @text_color.setter
    def text_color(self, value): ...
    @property
    def axis_color(self): ...
    @axis_color.setter
    def axis_color(self, value): ...
    @property
    def axis_width(self): ...
    @axis_width.setter
    def axis_width(self, value): ...
    @property
    def tick_color(self): ...
    @tick_color.setter
    def tick_color(self, value): ...
    @property
    def tick_width(self): ...
    @tick_width.setter
    def tick_width(self, value): ...
    @property
    def tick_font_size(self): ...
    @tick_font_size.setter
    def tick_font_size(self, value): ...
    @updating_property
    def tick_direction(self): ...
    @property
    def axis_font_size(self): ...
    @axis_font_size.setter
    def axis_font_size(self, value): ...
    @updating_property
    def domain(self): ...
    @updating_property
    def axis_label(self): ...
    @updating_property
    def pos(self): ...
    @updating_property
    def minor_tick_length(self): ...
    @updating_property
    def major_tick_length(self): ...
    @updating_property
    def tick_label_margin(self): ...
    @updating_property
    def axis_label_margin(self): ...
    @property
    def _vec(self): ...
    def _update_subvisuals(self): ...
    def _prepare_draw(self, view): ...
    @property
    def _rotation_angle(self): ...
    def _compute_bounds(self, axis, view): ...

class Ticker(object):
    def __init__(self, axis: AxisVisual, anchors=None): ...
    def get_update(self): ...
    def _get_tick_positions(self, major_tick_fractions, minor_tick_fractions): ...
    def _tile_ticks(self, frac, tickvec): ...
    def _get_tick_frac_labels(self): ...

# #############################################################################
# Translated from matplotlib

class MaxNLocator(object):
    def __init__(
        self,
        nbins=10,
        steps=None,
        trim=True,
        integer=False,
        symmetric=False,
        prune=None,
    ): ...
    def bin_boundaries(self, vmin, vmax): ...
    def __call__(self): ...
    def tick_values(self, vmin, vmax): ...
    def view_limits(self, dmin, dmax): ...

def scale_range(vmin, vmax, n=1, threshold=100): ...

# #############################################################################
# Tranlated from http://www.justintalbot.com/research/axis-labeling/

# See "An Extension of Wilkinson's Algorithm for Positioning Tick Labels
# on Axes" # by Justin Talbot, Sharon Lin, and Pat Hanrahan, InfoVis 2010.

def _coverage(dmin, dmax, lmin, lmax): ...
def _coverage_max(dmin, dmax, span): ...
def _density(k, m, dmin, dmax, lmin, lmax): ...
def _density_max(k, m): ...
def _simplicity(q, Q, j, lmin, lmax, lstep): ...
def _simplicity_max(q, Q, j): ...
def _get_ticks_talbot(dmin, dmax, n_inches, density=1.0): ...



from numpy import ndarray


class Bunch(dict):
    def __init__(self, **kwargs) -> None:
        ...

    def __setattr__(self, key: str, value: list[str] | ndarray | str) -> None:
        ...

    def __dir__(self) -> list[str]:
        ...

    def __getattr__(self, key: str):
        ...

    def __setstate__(self, state):
        ...



from typing import Type
from matplotlib.transforms import Bbox
from .backend_agg import FigureCanvasAgg
from ._backend_tk import FigureCanvasTk, _BackendTk

class FigureCanvasTkAgg(FigureCanvasAgg, FigureCanvasTk):
    def draw(self)-> None: ...
    def blit(self, bbox: Bbox = ...)-> None: ...

class _BackendTkAgg(_BackendTk):
    FigureCanvas: Type[FigureCanvasAgg] = FigureCanvasTkAgg



from ..classes.graph import Graph

from ..classes.graph import Graph
from ..utils import np_random_state

__all__ = ["spectral_graph_forge"]

@np_random_state(3)
def spectral_graph_forge(
    G: Graph, alpha: float, transformation: str = "identity", seed=None
) -> Graph: ...



from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration
from .._build_utils import gen_from_templates
import numpy


def configuration(parent_package: str = "", top_path=None):
    ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from ..util import _straight_line_vertices, _rescale_layout

class fruchterman_reingold(object):
    def __init__(
        self,
        optimal: float | None = None,
        iterations: int = 50,
        pos: ArrayLike | None = None,
    ): ...
    def __call__(self, adjacency_mat: ArrayLike, directed: bool = False): ...
    def _fruchterman_reingold(self, adjacency_mat, directed=False): ...
    def _sparse_fruchterman_reingold(self, adjacency_mat, directed=False): ...

def _calculate_delta_pos(adjacency_arr, pos, t, optimal): ...



from ._typing import *
from .font_manager import FontProperties

from types import SimpleNamespace

get_unicode_index = ...

class MathtextBackend:
    def __init__(self) -> None: ...
    def set_canvas_size(self, w: float, h: float, d: float) -> None: ...
    def render_glyph(self, ox: float, oy: float, info: SimpleNamespace): ...
    def render_rect_filled(self, x1: float, y1: float, x2: float, y2: float): ...
    def get_results(self, box) -> tuple: ...
    def get_hinting_type(self) -> int: ...

class MathtextBackendAgg(MathtextBackend):
    def __init__(self) -> None: ...
    def set_canvas_size(self, w: float, h: float, d: float) -> None: ...
    def render_glyph(self, ox: float, oy: float, info: SimpleNamespace) -> None: ...
    def render_rect_filled(
        self, x1: float, y1: float, x2: float, y2: float
    ) -> None: ...
    def get_results(self, box): ...
    def get_hinting_type(self) -> int: ...

class MathtextBackendPath(MathtextBackend):
    def __init__(self) -> None: ...
    def render_glyph(self, ox: float, oy: float, info: SimpleNamespace) -> None: ...
    def render_rect_filled(
        self, x1: float, y1: float, x2: float, y2: float
    ) -> None: ...
    def get_results(self, box): ...

class MathTextWarning(Warning): ...

class MathTextParser:
    def __init__(self, output: str) -> None: ...
    def parse(self, s: str, dpi: float = ..., prop: FontProperties = ...): ...

def math_to_image(
    s: str,
    filename_or_obj: str | PathLike | FileLike,
    prop: FontProperties = ...,
    dpi: float = ...,
    format: str = ...,
    *,
    color: str = ...
): ...



from numpy import dtype
from numpy.typing import NDArray, ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from ..color import Colormap

from ..gloo import Texture2D, VertexBuffer
from ..gloo.texture import should_cast_to_f32
from ..color import get_colormap
from .shaders import Function, FunctionChain
from .transforms import NullTransform
from .visual import Visual
from ..io import load_spatial_filters
from ._scalable_textures import CPUScaledTexture2D, GPUScaledTexture2D

_VERTEX_SHADER: str = ...

_FRAGMENT_SHADER: str = ...  # noqa

_INTERPOLATION_TEMPLATE: str = ...

_TEXTURE_LOOKUP: str = ...

_APPLY_CLIM_FLOAT: str = ...

_APPLY_CLIM: str = ...

_APPLY_GAMMA_FLOAT: str = ...

_APPLY_GAMMA: str = ...

_NULL_COLOR_TRANSFORM: str = ...

_C2L_RED: str = ...

_CUSTOM_FILTER: str = ...

class ImageVisual(Visual):

    _shaders: dict = ...

    _func_templates: dict = ...

    def __init__(
        self,
        data: NDArray | None = None,
        method: str = "auto",
        grid: tuple[float, float] = ...,
        cmap: str | Colormap = "viridis",
        clim: str | tuple = "auto",
        gamma: float = 1.0,
        interpolation: str = "nearest",
        texture_format: np.dtype | None | str = None,
        custom_kernel: NDArray = ...,
        **kwargs,
    ): ...
    def _init_interpolation(self, interpolation_names): ...
    def _init_texture(self, data, texture_format, **texture_kwargs): ...
    def set_data(self, image: ArrayLike): ...
    def view(self): ...
    def _init_view(self, view): ...
    @property
    def clim(self): ...
    @clim.setter
    def clim(self, clim): ...
    def _update_colortransform_clim(self): ...
    @property
    def cmap(self): ...
    @cmap.setter
    def cmap(self, cmap): ...
    @property
    def gamma(self): ...
    @gamma.setter
    def gamma(self, value): ...
    @property
    def method(self): ...
    @method.setter
    def method(self, m): ...
    @property
    def size(self): ...
    @property
    def interpolation(self): ...
    @interpolation.setter
    def interpolation(self, i): ...
    @property
    def interpolation_functions(self): ...
    @property
    def custom_kernel(self): ...
    @custom_kernel.setter
    def custom_kernel(self, value): ...

    # The interpolation code could be transferred to a dedicated filter
    # function in visuals/filters as discussed in #1051
    def _build_interpolation(self): ...
    def _build_vertex_data(self): ...
    def _update_method(self, view): ...
    def _build_texture(self): ...
    def _compute_bounds(self, axis, view): ...
    def _build_color_transform(self): ...
    def _prepare_transforms(self, view): ...
    def _prepare_draw(self, view): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.signal._peak_finding_utils, version: unspecified
import typing
import builtins as _mod_builtins

class PeakPropertyWarning(_mod_builtins.RuntimeWarning):
    'Calculated property of a peak has unexpected value.'
    __dict__: typing.Dict[str, typing.Any]
    def __init__(self, *args, **kwargs) -> None:
        'Calculated property of a peak has unexpected value.'
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __module__: str
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def __weakref__(self) -> typing.Any:
        'list of weak references to the object (if defined)'
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__all__: list
__doc__: str
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _local_maxima_1d() -> typing.Any:
    '\n    Find local maxima in a 1D array.\n\n    This function finds all local maxima in a 1D array and returns the indices\n    for their edges and midpoints (rounded down for even plateau sizes).\n\n    Parameters\n    ----------\n    x : ndarray\n        The array to search for local maxima.\n\n    Returns\n    -------\n    midpoints : ndarray\n        Indices of midpoints of local maxima in `x`.\n    left_edges : ndarray\n        Indices of edges to the left of local maxima in `x`.\n    right_edges : ndarray\n        Indices of edges to the right of local maxima in `x`.\n\n    Notes\n    -----\n    - Compared to `argrelmax` this function is significantly faster and can\n      detect maxima that are more than one sample wide. However this comes at\n      the cost of being only applicable to 1D arrays.\n    - A maxima is defined as one or more samples of equal value that are\n      surrounded on both sides by at least one smaller sample.\n\n    .. versionadded:: 1.1.0\n    '
    ...

def _peak_prominences() -> typing.Any:
    "\n    Calculate the prominence of each peak in a signal.\n\n    Parameters\n    ----------\n    x : ndarray\n        A signal with peaks.\n    peaks : ndarray\n        Indices of peaks in `x`.\n    wlen : np.intp\n        A window length in samples (see `peak_prominences`) which is rounded up\n        to the nearest odd integer. If smaller than 2 the entire signal `x` is\n        used.\n\n    Returns\n    -------\n    prominences : ndarray\n        The calculated prominences for each peak in `peaks`.\n    left_bases, right_bases : ndarray\n        The peaks' bases as indices in `x` to the left and right of each peak.\n\n    Raises\n    ------\n    ValueError\n        If a value in `peaks` is an invalid index for `x`.\n\n    Warns\n    -----\n    PeakPropertyWarning\n        If a prominence of 0 was calculated for any peak.\n\n    Notes\n    -----\n    This is the inner function to `peak_prominences`.\n\n    .. versionadded:: 1.1.0\n    "
    ...

def _peak_widths() -> typing.Any:
    "\n    Calculate the width of each each peak in a signal.\n\n    Parameters\n    ----------\n    x : ndarray\n        A signal with peaks.\n    peaks : ndarray\n        Indices of peaks in `x`.\n    rel_height : np.float64\n        Chooses the relative height at which the peak width is measured as a\n        percentage of its prominence (see `peak_widths`).\n    prominences : ndarray\n        Prominences of each peak in `peaks` as returned by `peak_prominences`.\n    left_bases, right_bases : ndarray\n        Left and right bases of each peak in `peaks` as returned by\n        `peak_prominences`.\n\n    Returns\n    -------\n    widths : ndarray\n        The widths for each peak in samples.\n    width_heights : ndarray\n        The height of the contour lines at which the `widths` where evaluated.\n    left_ips, right_ips : ndarray\n        Interpolated positions of left and right intersection points of a\n        horizontal line at the respective evaluation height.\n\n    Raises\n    ------\n    ValueError\n        If the supplied prominence data doesn't satisfy the condition\n        ``0 <= left_base <= peak <= right_base < x.shape[0]`` for each peak or\n        if `peaks`, `left_bases` and `right_bases` don't share the same shape.\n        Or if `rel_height` is not at least 0.\n\n    Warnings\n    --------\n    PeakPropertyWarning\n        If a width of 0 was calculated for any peak.\n\n    Notes\n    -----\n    This is the inner function to `peak_widths`.\n\n    .. versionadded:: 1.1.0\n    "
    ...

def _select_by_peak_distance() -> typing.Any:
    "\n    Evaluate which peaks fulfill the distance condition.\n\n    Parameters\n    ----------\n    peaks : ndarray\n        Indices of peaks in `vector`.\n    priority : ndarray\n        An array matching `peaks` used to determine priority of each peak. A\n        peak with a higher priority value is kept over one with a lower one.\n    distance : np.float64\n        Minimal distance that peaks must be spaced.\n\n    Returns\n    -------\n    keep : ndarray[bool]\n        A boolean mask evaluating to true where `peaks` fulfill the distance\n        condition.\n\n    Notes\n    -----\n    Declaring the input arrays as C-contiguous doesn't seem to have performance\n    advantages.\n\n    .. versionadded:: 1.1.0\n    "
    ...

def __getattr__(name) -> typing.Any:
    ...




from numpy.typing import ArrayLike
import gzip
import os
import os.path
from itertools import islice

from ..classes.graph import Graph

__all__ = ["graph_atlas", "graph_atlas_g"]

#: The total number of graphs in the atlas.
#:
#: The graphs are labeled starting from 0 and extending to (but not
#: including) this number.
NUM_GRAPHS: int = ...

#: The absolute path representing the directory containing this file.
THIS_DIR = ...

#: The path to the data file containing the graph edge lists.
#:
#: This is the absolute path of the gzipped text file containing the
#: edge list for each graph in the atlas. The file contains one entry
#: per graph in the atlas, in sequential order, starting from graph
#: number 0 and extending through graph number 1252 (see
#: :data:`NUM_GRAPHS`). Each entry looks like
#:
#: .. sourcecode:: text
#:
#:    GRAPH 6
#:    NODES 3
#:    0 1
#:    0 2
#:
#: where the first two lines are the graph's index in the atlas and the
#: number of nodes in the graph, and the remaining lines are the edge
#: list.
#:
#: This file was generated from a Python list of graphs via code like
#: the following::
#:
#:     import gzip
#:     from networkx.generators.atlas import graph_atlas_g
#:     from networkx.readwrite.edgelist import write_edgelist
#:
#:     with gzip.open('atlas.dat.gz', 'wb') as f:
#:         for i, G in enumerate(graph_atlas_g()):
#:             f.write(bytes(f'GRAPH {i}\n', encoding='utf-8'))
#:             f.write(bytes(f'NODES {len(G:Graph)}\n', encoding='utf-8'))
#:             write_edgelist(G:Graph, f, data=False)
#:
ATLAS_FILE = ...

def graph_atlas(i: int) -> ArrayLike: ...
def graph_atlas_g() -> ArrayLike: ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.magic import Magics, line_magic, magics_class
from IPython.core.magic_arguments import argument, magic_arguments
from IPython.testing.skipdoctest import skip_doctest

"""Implementation of magic functions related to History.
"""
_unspecified = ...
@magics_class
class HistoryMagics(Magics):
    @magic_arguments()
    @argument('-n', dest='print_nums', action='store_true', default=False, help="""
        print line numbers for each input.
        This feature is only available if numbered prompts are in use.
        """)
    @argument('-o', dest='get_output', action='store_true', default=False, help="also print outputs for each input.")
    @argument('-p', dest='pyprompts', action='store_true', default=False, help="""
        print classic '>>>' python prompts before each input.
        This is useful for making documentation, and in conjunction
        with -o, for producing doctest-ready output.
        """)
    @argument('-t', dest='raw', action='store_false', default=True, help="""
        print the 'translated' history, as IPython understands it.
        IPython filters your input and converts it all into valid Python
        source before executing it (things like magics or aliases are turned
        into function calls, for example). With this option, you'll see the
        native history instead of the user-entered version: '%%cd /' will be
        seen as 'get_ipython().run_line_magic("cd", "/")' instead of '%%cd /'.
        """)
    @argument('-f', dest='filename', help="""
        FILENAME: instead of printing the output to the screen, redirect
        it to the given file.  The file is always overwritten, though *when
        it can*, IPython asks for confirmation first. In particular, running
        the command 'history -f FILENAME' from the IPython Notebook
        interface will replace FILENAME even if it already exists *without*
        confirmation.
        """)
    @argument('-g', dest='pattern', nargs='*', default=None, help="""
        treat the arg as a glob pattern to search for in (full) history.
        This includes the saved history (almost all commands ever written).
        The pattern may contain '?' to match one unknown character and '*'
        to match any number of unknown characters. Use '%%hist -g' to show
        full saved history (may be very long).
        """)
    @argument('-l', dest='limit', type=int, nargs='?', default=_unspecified, help="""
        get the last n lines from all sessions. Specify n as a single
        arg, or the default is the last 10 lines.
        """)
    @argument('-u', dest='unique', action='store_true', help="""
        when searching history using `-g`, show only unique history.
        """)
    @argument('range', nargs='*')
    @skip_doctest
    @line_magic
    def history(self, parameter_s=...): # -> None:
        """Print input history (_i<n> variables), with most recent last.

        By default, input history is printed without line numbers so it can be
        directly pasted into an editor. Use -n to show them.

        By default, all input history from the current session is displayed.
        Ranges of history can be indicated using the syntax:

        ``4``
            Line 4, current session
        ``4-6``
            Lines 4-6, current session
        ``243/1-5``
            Lines 1-5, session 243
        ``~2/7``
            Line 7, session 2 before current
        ``~8/1-~6/5``
            From the first line of 8 sessions ago, to the fifth line of 6
            sessions ago.

        Multiple ranges can be entered, separated by spaces

        The same syntax is used by %macro, %save, %edit, %rerun

        Examples
        --------
        ::

          In [6]: %history -n 4-6
          4:a = 12
          5:print a**2
          6:%history -n 4-6

        """
        ...
    
    @line_magic
    def recall(self, arg): # -> None:
        r"""Repeat a command, or get command to input line for editing.

        %recall and %rep are equivalent.

        - %recall (no arguments):

        Place a string version of last computation result (stored in the
        special '_' variable) to the next input prompt. Allows you to create
        elaborate command lines without using copy-paste::

             In[1]: l = ["hei", "vaan"]
             In[2]: "".join(l)
            Out[2]: heivaan
             In[3]: %recall
             In[4]: heivaan_ <== cursor blinking

        %recall 45

        Place history line 45 on the next input prompt. Use %hist to find
        out the number.

        %recall 1-4

        Combine the specified lines into one cell, and place it on the next
        input prompt. See %history for the slice syntax.

        %recall foo+bar

        If foo+bar can be evaluated in the user namespace, the result is
        placed at the next input prompt. Otherwise, the history is searched
        for lines which contain that substring, and the most recent one is
        placed at the next input prompt.
        """
        ...
    
    @line_magic
    def rerun(self, parameter_s=...): # -> None:
        """Re-run previous input

        By default, you can specify ranges of input history to be repeated
        (as with %history). With no arguments, it will repeat the last line.

        Options:

          -l <n> : Repeat the last n lines of input, not including the
          current command.

          -g foo : Repeat the most recent line which contains foo
        """
        ...
    





from .loss import (
    HalfSquaredError as HalfSquaredError,
    AbsoluteError as AbsoluteError,
    PinballLoss as PinballLoss,
    HalfPoissonLoss as HalfPoissonLoss,
    HalfGammaLoss as HalfGammaLoss,
    HalfTweedieLoss as HalfTweedieLoss,
    HalfTweedieLossIdentity as HalfTweedieLossIdentity,
    HalfBinomialLoss as HalfBinomialLoss,
    HalfMultinomialLoss as HalfMultinomialLoss,
)


__all__ = [
    "HalfSquaredError",
    "AbsoluteError",
    "PinballLoss",
    "HalfPoissonLoss",
    "HalfGammaLoss",
    "HalfTweedieLoss",
    "HalfTweedieLossIdentity",
    "HalfBinomialLoss",
    "HalfMultinomialLoss",
]



from ...classes.graph import Graph
from ...utils.decorators import not_implemented_for

from ...utils import arbitrary_element

__all__ = [
    "number_connected_components",
    "connected_components",
    "is_connected",
    "node_connected_component",
]

def connected_components(G: Graph): ...
def number_connected_components(G: Graph): ...
def is_connected(G: Graph) -> bool: ...
def node_connected_component(G: Graph, n) -> set: ...



import numpy as np
from typing import Any
from matplotlib._typing import *
from .trifinder import TrapezoidMapTriFinder

class Triangulation:
    def __init__(
        self,
        x: ArrayLike,
        y: ArrayLike,
        triangles: ArrayLike = ...,
        mask: ArrayLike = ...,
    ) -> None: ...
    def calculate_plane_coefficients(self, z: ArrayLike) -> np.ndarray: ...
    @property
    def edges(self) -> np.ndarray: ...
    def get_cpp_triangulation(self) -> object: ...
    def get_masked_triangles(self) -> np.ndarray: ...
    @staticmethod
    def get_from_args_and_kwargs(
        *args, **kwargs
    ) -> tuple[Triangulation, list, dict[str, Any]]: ...
    def get_trifinder(self) -> TrapezoidMapTriFinder: ...
    @property
    def neighbors(self) -> np.ndarray: ...
    def set_mask(self, mask: None) -> None: ...



from numpy.typing import NDArray
import numpy as np
from scipy import ndimage as ndi

from .._shared.utils import deprecate_kwarg
from .misc import default_footprint

# The default_footprint decorator provides a diamond footprint as
# default with the same dimension as the input image and size 3 along each
# axis.
@default_footprint
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def binary_erosion(
    image: NDArray, footprint: NDArray | None = None, out: NDArray | None = None
): ...
@default_footprint
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def binary_dilation(
    image: NDArray, footprint: NDArray | None = None, out: NDArray | None = None
): ...
@default_footprint
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def binary_opening(
    image: NDArray, footprint: NDArray | None = None, out: NDArray | None = None
) -> NDArray: ...
@default_footprint
@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def binary_closing(
    image: NDArray, footprint: NDArray | None = None, out: NDArray | None = None
) -> NDArray: ...



from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import numpy as np

from .mesh import MeshVisual
from ..geometry import MeshData

class SurfacePlotVisual(MeshVisual):
    def __init__(
        self,
        x: NDArray | None = None,
        y: NDArray | None = None,
        z: NDArray | None = None,
        colors: NDArray | None = None,
        **kwargs,
    ): ...
    def _update_x_data(self, x): ...
    def _update_y_data(self, y): ...
    def _update_z_data(self, z): ...
    def _update_mesh_vertices(self, x_is_new, y_is_new, z_is_new): ...
    def _prepare_mesh_colors(self, colors): ...
    def set_data(
        self,
        x: NDArray | None = None,
        y: NDArray | None = None,
        z: NDArray | None = None,
        colors: NDArray | None = None,
    ): ...
    def _generate_faces(self): ...



from vispy.scene.node import Node
from typing import Literal, Mapping
from vispy.util.event import Event
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ...util import keys
from ..node import Node
from ...visuals.transforms import (
    STTransform,
    MatrixTransform,
    NullTransform,
    TransformCache,
)

def nested_getattr(obj, names): ...
def nested_setattr(obj, names, val): ...

class BaseCamera(Node):

    # These define the state of the camera
    _state_props = ...

    # The fractional zoom to apply for a single pixel of mouse motion
    zoom_factor: float = ...

    def __init__(
        self,
        interactive: bool = True,
        flip: tuple[bool, ...] | None = None,
        up: Literal["+z", "-z", "+y", "-y", "+x", "-x"] = "+z",
        parent: Node | None = None,
        name: str | None = None,
    ): ...
    @property
    def depth_value(self): ...
    @depth_value.setter
    def depth_value(self, value): ...
    def _depth_to_z(self, depth): ...
    def _viewbox_set(self, viewbox): ...
    def _viewbox_unset(self, viewbox): ...
    @property
    def viewbox(self): ...

    # Camera attributes

    @property
    def interactive(self): ...
    @interactive.setter
    def interactive(self, value): ...
    @property
    def flip(self): ...
    @flip.setter
    def flip(self, value): ...
    @property
    def up(self): ...
    @up.setter
    def up(self, value): ...
    @property
    def center(self): ...
    @center.setter
    def center(self, val): ...
    @property
    def fov(self): ...
    @fov.setter
    def fov(self, fov): ...

    # Camera methods

    def set_range(
        self,
        x: tuple | None | None = None,
        y: tuple | None | None = None,
        z: tuple | None | None = None,
        margin: float = 0.05,
    ): ...
    def _set_range(self, init): ...
    def reset(self): ...
    def set_default_state(self): ...
    def get_state(self, props: ArrayLike | None = None): ...
    def set_state(self, state: Mapping | None = None, **kwargs): ...
    def link(
        self,
        camera: BaseCamera,
        props: ArrayLike | None = None,
        axis: Literal["y", "x"] | None = None,
    ): ...

    # Event-related methods

    def view_changed(self): ...
    @property
    def pre_transform(self): ...
    @pre_transform.setter
    def pre_transform(self, tr): ...
    def viewbox_mouse_event(self, event: Event): ...
    def on_canvas_change(self, event: Event): ...
    def viewbox_key_event(self, event: Event): ...
    def viewbox_resize_event(self, event: Event): ...
    def _update_transform(self): ...
    def _set_scene_transform(self, tr): ...



from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from ..util import _straight_line_vertices, issparse

def circular(adjacency_mat: ArrayLike | NDArray, directed: bool = False): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.io.matlab.streams, version: unspecified
import typing
import builtins as _mod_builtins

BLOCK_SIZE: int
class GenericStream(_mod_builtins.object):
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __pyx_vtable__: PyCapsule
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def all_data_read(self) -> typing.Any:
        ...
    
    def read(self) -> typing.Any:
        ...
    
    def seek(self) -> typing.Any:
        ...
    
    def tell(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class ZlibInputStream(GenericStream):
    '\n    File-like object uncompressing bytes from a zlib compressed stream.\n\n    Parameters\n    ----------\n    stream : file-like\n        Stream to read compressed data from.\n    max_length : int\n        Maximum number of bytes to read from the stream.\n\n    Notes\n    -----\n    Some matlab files contain zlib streams without valid Z_STREAM_END\n    termination.  To get round this, we use the decompressobj object, that\n    allows you to decode an incomplete stream.  See discussion at\n    https://bugs.python.org/issue8672\n\n    '
    def __init__(self, *args, **kwargs) -> None:
        '\n    File-like object uncompressing bytes from a zlib compressed stream.\n\n    Parameters\n    ----------\n    stream : file-like\n        Stream to read compressed data from.\n    max_length : int\n        Maximum number of bytes to read from the stream.\n\n    Notes\n    -----\n    Some matlab files contain zlib streams without valid Z_STREAM_END\n    termination.  To get round this, we use the decompressobj object, that\n    allows you to decode an incomplete stream.  See discussion at\n    https://bugs.python.org/issue8672\n\n    '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __pyx_vtable__: PyCapsule
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def all_data_read(self) -> typing.Any:
        ...
    
    def read(self) -> typing.Any:
        ...
    
    def seek(self) -> typing.Any:
        ...
    
    def tell(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__pyx_capi__: dict
def __pyx_unpickle_GenericStream() -> typing.Any:
    ...

def __pyx_unpickle_ZlibInputStream() -> typing.Any:
    ...

__test__: dict
def _read_into() -> typing.Any:
    ...

def _read_string() -> typing.Any:
    ...

def make_stream() -> typing.Any:
    ' Make stream of correct type for file-like `fobj`\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




import itertools as it
from functools import partial

from ...classes.graph import Graph
from ...algorithms import bridges
from ...utils import arbitrary_element, not_implemented_for

__all__ = [
    "k_edge_components",
    "k_edge_subgraphs",
    "bridge_components",
    "EdgeComponentAuxGraph",
]

def k_edge_components(G: Graph, k): ...
def k_edge_subgraphs(G: Graph, k): ...
def bridge_components(G: Graph): ...

class EdgeComponentAuxGraph:
    @classmethod
    def construct(cls, G: Graph): ...
    def k_edge_components(self, k): ...
    def k_edge_subgraphs(self, k): ...

def general_k_edge_subgraphs(G: Graph, k): ...



from typing import Callable, ClassVar, TypeVar
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    check_consistent_length as check_consistent_length,
)
from abc import ABCMeta, abstractmethod
from ..exceptions import (
    ConvergenceWarning as ConvergenceWarning,
    NotFittedError as NotFittedError,
)
from numpy import ndarray
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from numpy.random.mtrand import RandomState
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from ..utils.metaestimators import available_if as available_if
from ..base import BaseEstimator, ClassifierMixin
from .._typing import Float, MatrixLike, ArrayLike
from ..utils import (
    check_array as check_array,
    check_random_state as check_random_state,
    column_or_1d as column_or_1d,
    compute_class_weight as compute_class_weight,
)
from ..preprocessing import LabelEncoder as LabelEncoder

BaseLibSVM_Self = TypeVar("BaseLibSVM_Self", bound="BaseLibSVM")

import warnings

import numpy as np
import scipy.sparse as sp


LIBSVM_IMPL: list = ...


class BaseLibSVM(BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    # The order of these must match the integer values in LibSVM.
    # XXX These are actually the same in the dense case. Need to factor
    # this out.
    _sparse_kernels: ClassVar[list] = ...

    @abstractmethod
    def __init__(
        self,
        kernel: str | Callable,
        degree: int,
        gamma: str | Float,
        coef0: float | int,
        tol: float,
        C: Float,
        nu: float,
        epsilon: float,
        shrinking: bool,
        probability: bool,
        cache_size: int,
        class_weight: dict[int, int] | None | str,
        verbose: bool,
        max_iter: int,
        random_state: None | RandomState | int,
    ) -> None:
        ...

    def fit(
        self: BaseLibSVM_Self,
        X: MatrixLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> BaseLibSVM_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    @property
    def coef_(self) -> ndarray:
        ...

    def n_support_(self) -> ndarray:
        ...


class BaseSVC(ClassifierMixin, BaseLibSVM, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...
    for unused_param in ["epsilon", "nu"]:
        pass

    @abstractmethod
    def __init__(
        self,
        kernel: str | Callable,
        degree: int,
        gamma: str | Float,
        coef0: float | int,
        tol: float,
        C: Float,
        nu: float,
        shrinking: bool,
        probability: bool,
        cache_size: int,
        class_weight: dict[int, int] | None | str,
        verbose: bool,
        max_iter: int,
        decision_function_shape: str,
        random_state: None | RandomState | int,
        break_ties: bool,
    ) -> None:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike) -> ndarray:
        ...

    @property
    def probA_(self) -> ndarray:
        ...

    @property
    def probB_(self) -> ndarray:
        ...



from typing import Any, Callable, Iterable, Iterator
from numpy import ndarray
from .._config import config_context as config_context, get_config as get_config
from functools import update_wrapper as update_wrapper

import functools
import warnings

import joblib


class Parallel(joblib.Parallel):
    def __call__(self, iterable: Iterable | Iterator[Any]) -> ndarray:
        ...


# remove when https://github.com/joblib/joblib/issues/1071 is fixed
def delayed(function: Callable) -> tuple | Callable:
    ...


class _FuncWrapper:
    def __init__(self, function: Callable) -> None:
        ...

    def with_config(self, config: dict[str, bool | int | str]) -> _FuncWrapper:
        ...

    def __call__(self, *args, **kwargs):
        ...



from typing import Literal
from .gridspec import SubplotSpec
from .transforms import Bbox
from .figure import Figure

class LayoutGrid:
    def __init__(
        self,
        parent: LayoutGrid | tuple[int, int, int, int]|None = None,
        parent_pos: tuple[range, range] | tuple[int, int] = (0, 0),
        parent_inner: bool = False,
        name: str = '',
        ncols: int = 1,
        nrows: int = 1,
        h_pad: float|None = None,
        w_pad: float|None = None,
        width_ratios: None | list[float] = None,
        height_ratios: None | list[float] = None,
    ) -> None: ...
    def __repr__(self) -> str: ...
    def reset_margins(self) -> None: ...
    def add_constraints(self) -> None: ...
    def hard_constraints(self) -> None: ...
    def add_child(
        self,
        child: LayoutGrid,
        i: int | range = 0,
        j: int | range = 0,
    ) -> None: ...
    def parent_constraints(self) -> None: ...
    def grid_constraints(self) -> None: ...
    def edit_margin(
        self, todo: Literal["left", "right", "bottom", "top"], size: float, cell: int
    ) -> None: ...
    def edit_margin_min(
        self,
        todo: Literal["left", "right", "bottom", "top"],
        size: float,
        cell: int = 0,
    ) -> None: ...
    def edit_margins(
        self, todo: Literal["left", "right", "bottom", "top"], size: float
    ) -> None: ...
    def edit_all_margins_min(
        self, todo: Literal["left", "right", "bottom", "top"], size: float
    ): ...
    def edit_outer_margin_mins(self, margin: dict, ss: SubplotSpec)-> None: ...
    def get_margins(self, todo, col) -> Bbox: ...
    def get_outer_bbox(self, rows=0, cols=0) -> Bbox: ...
    def get_inner_bbox(self, rows=0, cols=0) -> Bbox: ...
    def get_bbox_for_cb(self, rows=0, cols=0) -> Bbox: ...
    def get_left_margin_bbox(self, rows=0, cols=0) -> Bbox: ...
    def get_bottom_margin_bbox(self, rows=0, cols=0) -> Bbox: ...
    def get_right_margin_bbox(self, rows=0, cols=0) -> Bbox: ...
    def get_top_margin_bbox(self, rows=0, cols=0) -> Bbox: ...
    def update_variables(self) -> None: ...

def seq_id() -> str: ...
def print_children(lb) -> None: ...
def plot_children(fig: Figure, lg=None, level: int=0, printit: bool=False)->None: ...



from ._twenty_newsgroups import (
    fetch_20newsgroups as fetch_20newsgroups,
    fetch_20newsgroups_vectorized as fetch_20newsgroups_vectorized,
)
from ._svmlight_format_io import (
    load_svmlight_file as load_svmlight_file,
    load_svmlight_files as load_svmlight_files,
    dump_svmlight_file as dump_svmlight_file,
)
from ._samples_generator import (
    make_classification as make_classification,
    make_multilabel_classification as make_multilabel_classification,
    make_hastie_10_2 as make_hastie_10_2,
    make_regression as make_regression,
    make_blobs as make_blobs,
    make_moons as make_moons,
    make_circles as make_circles,
    make_friedman1 as make_friedman1,
    make_friedman2 as make_friedman2,
    make_friedman3 as make_friedman3,
    make_low_rank_matrix as make_low_rank_matrix,
    make_sparse_coded_signal as make_sparse_coded_signal,
    make_sparse_uncorrelated as make_sparse_uncorrelated,
    make_spd_matrix as make_spd_matrix,
    make_swiss_roll as make_swiss_roll,
    make_s_curve as make_s_curve,
    make_sparse_spd_matrix as make_sparse_spd_matrix,
    make_gaussian_quantiles as make_gaussian_quantiles,
    make_biclusters as make_biclusters,
    make_checkerboard as make_checkerboard,
)
from ._base import (
    load_breast_cancer as load_breast_cancer,
    load_diabetes as load_diabetes,
    load_digits as load_digits,
    load_files as load_files,
    load_iris as load_iris,
    load_linnerud as load_linnerud,
    load_sample_images as load_sample_images,
    load_sample_image as load_sample_image,
    load_wine as load_wine,
    get_data_home as get_data_home,
    clear_data_home as clear_data_home,
)
from ._lfw import (
    fetch_lfw_pairs as fetch_lfw_pairs,
    fetch_lfw_people as fetch_lfw_people,
)
from ._kddcup99 import fetch_kddcup99 as fetch_kddcup99
from ._covtype import fetch_covtype as fetch_covtype
from ._openml import fetch_openml as fetch_openml
from ._california_housing import fetch_california_housing as fetch_california_housing
from ._olivetti_faces import fetch_olivetti_faces as fetch_olivetti_faces
from ._species_distributions import (
    fetch_species_distributions as fetch_species_distributions,
)
from ._rcv1 import fetch_rcv1 as fetch_rcv1
import textwrap as textwrap


__all__ = [
    "clear_data_home",
    "dump_svmlight_file",
    "fetch_20newsgroups",
    "fetch_20newsgroups_vectorized",
    "fetch_lfw_pairs",
    "fetch_lfw_people",
    "fetch_olivetti_faces",
    "fetch_species_distributions",
    "fetch_california_housing",
    "fetch_covtype",
    "fetch_rcv1",
    "fetch_kddcup99",
    "fetch_openml",
    "get_data_home",
    "load_diabetes",
    "load_digits",
    "load_files",
    "load_iris",
    "load_breast_cancer",
    "load_linnerud",
    "load_sample_image",
    "load_sample_images",
    "load_svmlight_file",
    "load_svmlight_files",
    "load_wine",
    "make_biclusters",
    "make_blobs",
    "make_circles",
    "make_classification",
    "make_checkerboard",
    "make_friedman1",
    "make_friedman2",
    "make_friedman3",
    "make_gaussian_quantiles",
    "make_hastie_10_2",
    "make_low_rank_matrix",
    "make_moons",
    "make_multilabel_classification",
    "make_regression",
    "make_s_curve",
    "make_sparse_coded_signal",
    "make_sparse_spd_matrix",
    "make_sparse_uncorrelated",
    "make_spd_matrix",
    "make_swiss_roll",
]



from typing import Literal

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
import numpy as np
from .shader_object import ShaderObject

VARIABLE_TYPES = ...

class Variable(ShaderObject):

    _vtype_32_conversion: dict = ...
    _vtype_23_conversion: dict = ...

    def __init__(
        self,
        name: str,
        value: float | int | tuple | GLObject | None = None,
        vtype: Literal["const", "uniform", "attribute", "varying", "inout"] | None = None,
        dtype: str | None = None,
    ): ...
    @property
    def name(self): ...
    @name.setter
    def name(self, n): ...
    @property
    def vtype(self): ...
    @property
    def dtype(self): ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value): ...
    @property
    def state_id(self): ...
    def __repr__(self): ...
    def expression(self, names): ...
    def _vtype_for_version(self, version): ...
    def definition(self, names, version, shader): ...

class Varying(Variable):
    def __init__(self, name, dtype=None): ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value): ...
    @property
    def dtype(self): ...
    def link(self, var): ...
    def invar(self, array=False): ...

class InVar(Variable):
    def __init__(self, var, array=False): ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value): ...
    @property
    def dtype(self): ...
    def definition(self, names, version, shader): ...
    def expression(self, names): ...



import numpy as np
from typing import Callable, Literal
from matplotlib._typing import *
from matplotlib.axes._base import _AxesBase
from matplotlib.transforms import Transform

class SecondaryAxis(_AxesBase):
    def __init__(self, parent, orientation, location, functions, **kwargs) -> None: ...
    def set_alignment(
        self, align: Literal["top", "bottom", "left", "right"]
    ) -> None: ...
    def set_location(
        self, location: Literal["top", "bottom", "left", "right"] | float
    ) -> None: ...
    def apply_aspect(self, position=None) -> None: ...
    def set_ticks(
        self,
        ticks: list[float],
        labels: list[str]|None = None,
        *,
        minor: bool = False,
        **kwargs
    ) -> list: ...
    def set_functions(
        self, functions: tuple[Callable, Callable] | Transform
    ) -> None: ...
    def draw(self, *args, **kwargs) -> None: ...
    def set_aspect(self, *args, **kwargs) -> None: ...
    def set_color(self, color: Color) -> None: ...



import json
from typing import (
    Any,
    Callable,
    Iterable,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

from django.db.models import lookups
from django.db.models.expressions import Combinable
from django.db.models.lookups import PostgresOperatorLookup, Transform
from typing_extensions import Literal

from . import Field, _ErrorMessagesToOverride, _ValidatorCallable
from .mixins import CheckFieldDefaultMixin

_A = TypeVar("_A", bound=Optional[Any])

class JSONField(CheckFieldDefaultMixin, Field[Union[_A, Combinable], _A]):
    default_error_messages: Any = ...
    encoder: Type[json.JSONEncoder] = ...
    decoder: Type[json.JSONEncoder] = ...
    def from_db_value(self, value: Any, expression: Any, connection: Any) -> Any: ...
    def get_transform(self, name: Any) -> Any: ...
    def value_to_string(self, obj: Any) -> Any: ...
    @overload
    def __new__(  # type: ignore [misc]
        cls,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        encoder: Type[json.JSONEncoder] = ...,
        decoder: Type[json.JSONDecoder] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Optional[Union[Any, Callable[[], Any]]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[Any, str], Tuple[str, Iterable[Tuple[Any, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> JSONField[_A]: ...
    @overload
    def __new__(
        cls,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        encoder: Type[json.JSONEncoder] = ...,
        decoder: Type[json.JSONDecoder] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Union[Any, Callable[[], Any]] = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Iterable[
            Union[Tuple[Any, str], Tuple[str, Iterable[Tuple[Any, str]]]]
        ] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> JSONField[Optional[_A]]: ...

class DataContains(PostgresOperatorLookup):
    lookup_name: str = ...
    postgres_operator: str = ...
    def as_sql(self, compiler: Any, connection: Any) -> Any: ...

class ContainedBy(PostgresOperatorLookup):
    lookup_name: str = ...
    postgres_operator: str = ...
    def as_sql(self, compiler: Any, connection: Any) -> Any: ...

class HasKeyLookup(PostgresOperatorLookup):
    logical_operator: Any = ...
    def as_sql(
        self, compiler: Any, connection: Any, template: Optional[Any] = ...
    ) -> Any: ...
    def as_mysql(self, compiler: Any, connection: Any) -> Any: ...
    def as_oracle(self, compiler: Any, connection: Any) -> Any: ...
    lhs: Any = ...
    rhs: Any = ...
    def as_postgresql(self, compiler: Any, connection: Any) -> Any: ...
    def as_sqlite(self, compiler: Any, connection: Any) -> Any: ...

class HasKey(HasKeyLookup):
    lookup_name: str = ...
    postgres_operator: str = ...
    prepare_rhs: bool = ...

class HasKeys(HasKeyLookup):
    lookup_name: str = ...
    postgres_operator: str = ...
    logical_operator: str = ...
    def get_prep_lookup(self) -> Any: ...

class HasAnyKeys(HasKeys):
    lookup_name: str = ...
    postgres_operator: str = ...
    logical_operator: str = ...

class JSONExact(lookups.Exact):
    can_use_none_as_rhs: bool = ...
    def process_rhs(self, compiler: Any, connection: Any) -> Any: ...

class KeyTransform(Transform):
    postgres_operator: str = ...
    postgres_nested_operator: str = ...
    key_name: Any = ...
    def __init__(self, key_name: Any, *args: Any, **kwargs: Any) -> None: ...
    def preprocess_lhs(
        self, compiler: Any, connection: Any, lhs_only: bool = ...
    ) -> Any: ...
    def as_mysql(self, compiler: Any, connection: Any) -> Any: ...
    def as_oracle(self, compiler: Any, connection: Any) -> Any: ...
    def as_postgresql(self, compiler: Any, connection: Any) -> Any: ...

class KeyTextTransform(KeyTransform):
    postgres_operator: str = ...
    postgres_nested_operator: str = ...

class KeyTransformTextLookupMixin:
    def __init__(self, key_transform: Any, *args: Any, **kwargs: Any) -> None: ...

class CaseInsensitiveMixin:
    def process_rhs(self, compiler: Any, connection: Any) -> Any: ...

class KeyTransformIsNull(lookups.IsNull):
    def as_oracle(self, compiler: Any, connection: Any) -> Any: ...
    def as_sqlite(self, compiler: Any, connection: Any) -> Any: ...

class KeyTransformIn(lookups.In):
    def process_rhs(self, compiler: Any, connection: Any) -> Any: ...

class KeyTransformExact(JSONExact):
    def process_rhs(self, compiler: Any, connection: Any) -> Any: ...
    def as_oracle(self, compiler: Any, connection: Any) -> Any: ...

class KeyTransformIExact(
    CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IExact
): ...
class KeyTransformIContains(
    CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IContains
): ...
class KeyTransformStartsWith(KeyTransformTextLookupMixin, lookups.StartsWith): ...
class KeyTransformIStartsWith(
    CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IStartsWith
): ...
class KeyTransformEndsWith(KeyTransformTextLookupMixin, lookups.EndsWith): ...
class KeyTransformIEndsWith(
    CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IEndsWith
): ...
class KeyTransformRegex(KeyTransformTextLookupMixin, lookups.Regex): ...
class KeyTransformIRegex(
    CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IRegex
): ...

class KeyTransformNumericLookupMixin:
    def process_rhs(self, compiler: Any, connection: Any) -> Any: ...

class KeyTransformLt(KeyTransformNumericLookupMixin, lookups.LessThan[Any]): ...
class KeyTransformLte(KeyTransformNumericLookupMixin, lookups.LessThanOrEqual): ...
class KeyTransformGt(KeyTransformNumericLookupMixin, lookups.GreaterThan): ...
class KeyTransformGte(
    KeyTransformNumericLookupMixin, lookups.GreaterThanOrEqual[Any]
): ...

class KeyTransformFactory:
    key_name: Any = ...
    def __init__(self, key_name: Any) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...

KT: KeyTextTransform = ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ..util import SimpleBunch
import time
from timeit import default_timer

class BaseApplicationBackend(object):
    def _vispy_get_backend_name(self): ...
    def _vispy_process_events(self): ...
    def _vispy_run(self): ...
    def _vispy_reuse(self): ...
    def _vispy_quit(self): ...
    def _vispy_get_native_app(self): ...

    # is called by inputhook.py for pauses
    # to remove CPU stress
    # this is virtual so that some backends which have specialize
    # functionality to deal with user input / latency can use those methods
    def _vispy_sleep(self, duration_sec): ...

class BaseCanvasBackend(object):
    def __init__(self, vispy_canvas): ...
    def _process_backend_kwargs(self, kwargs): ...
    def _vispy_set_current(self): ...
    def _vispy_swap_buffers(self): ...
    def _vispy_set_title(self, title): ...
    def _vispy_set_size(self, w, h): ...
    def _vispy_set_position(self, x, y): ...
    def _vispy_set_visible(self, visible): ...
    def _vispy_set_fullscreen(self, fullscreen): ...
    def _vispy_update(self): ...
    def _vispy_close(self): ...
    def _vispy_get_size(self): ...
    def _vispy_get_physical_size(self): ...
    def _vispy_get_position(self): ...
    def _vispy_get_fullscreen(self): ...
    def _vispy_get_geometry(self): ...
    def _vispy_get_native_canvas(self): ...
    def _vispy_get_fb_bind_location(self): ...
    def _vispy_mouse_press(self, **kwargs): ...
    def _vispy_mouse_move(self, **kwargs): ...
    def _vispy_mouse_release(self, **kwargs): ...
    def _vispy_mouse_double_click(self, **kwargs): ...
    def _vispy_detect_double_click(self, ev, **kwargs): ...

class BaseTimerBackend(object):
    def __init__(self, vispy_timer): ...
    def _vispy_start(self, interval): ...
    def _vispy_stop(self): ...
    def _vispy_get_native_timer(self): ...



from ..classes.graph import Graph
from ..exception import NetworkXError

__all__ = ["hnm_harary_graph", "hkn_harary_graph"]

def hnm_harary_graph(n, m, create_using=None): ...
def hkn_harary_graph(k, n, create_using=None): ...



from typing import Literal, Sequence
from numpy.random import RandomState
from collections import namedtuple as namedtuple
from os import environ as environ, listdir as listdir, makedirs as makedirs
from ..utils._bunch import Bunch
from os.path import (
    expanduser as expanduser,
    isdir as isdir,
    join as join,
    splitext as splitext,
)
from numpy import ndarray
from urllib.request import urlretrieve as urlretrieve
from pathlib import Path as Path
from PIL import Image as Image
from importlib import resources as resources
from .._typing import Int
from ..utils import (
    check_random_state as check_random_state,
    check_pandas_support as check_pandas_support,
)
from ..preprocessing import scale as scale

# Copyright (c) 2007 David Cournapeau <cournape@gmail.com>
#               2010 Fabian Pedregosa <fabian.pedregosa@inria.fr>
#               2010 Olivier Grisel <olivier.grisel@ensta.org>
# License: BSD 3 clause
import csv
import hashlib
import gzip
import shutil
import os

import numpy as np

DATA_MODULE: str = ...
DESCR_MODULE: str = ...
IMAGES_MODULE: str = ...

RemoteFileMetadata = ...


def get_data_home(data_home: None | str = None) -> str:
    ...


def clear_data_home(data_home: None | str = None):
    ...


def load_files(
    container_path: str,
    *,
    description: None | str = None,
    categories: None | Sequence[str] = None,
    load_content: bool = True,
    shuffle: bool = True,
    encoding: None | str = None,
    decode_error: Literal["strict", "ignore", "replace", "strict"] = "strict",
    random_state: RandomState | None | Int = 0,
    allowed_extensions: None | Sequence[str] = None,
) -> Bunch:
    ...


def load_csv_data(
    data_file_name: str,
    *,
    data_module: str = ...,
    descr_file_name: None | str = None,
    descr_module: str = ...,
) -> tuple[ndarray, ndarray, ndarray, str]:
    ...


def load_gzip_compressed_csv_data(
    data_file_name: str,
    *,
    data_module: str = ...,
    descr_file_name: None | str = None,
    descr_module: str = ...,
    encoding: str = "utf-8",
    **kwargs,
) -> ndarray | tuple[ndarray, str]:
    ...


def load_descr(descr_file_name: str, *, descr_module: str = ...) -> str:
    ...


def load_wine(
    *, return_X_y: bool = False, as_frame: bool = False
) -> tuple[Bunch, tuple]:
    ...


def load_iris(
    *, return_X_y: bool = False, as_frame: bool = False
) -> tuple[Bunch, tuple]:
    ...


def load_breast_cancer(
    *, return_X_y: bool = False, as_frame: bool = False
) -> Bunch | tuple[Bunch, tuple] | tuple[ndarray, ndarray]:
    ...


def load_digits(
    *, n_class: Int = 10, return_X_y: bool = False, as_frame: bool = False
) -> Bunch | tuple[Bunch, tuple] | tuple[ndarray, ndarray]:
    ...


def load_diabetes(
    *, return_X_y: bool = False, as_frame: bool = False, scaled: bool = True
) -> tuple[Bunch, tuple]:
    ...


def load_linnerud(
    *, return_X_y: bool = False, as_frame: bool = False
) -> tuple[Bunch, tuple]:
    ...


def load_sample_images() -> Bunch:
    ...


def load_sample_image(image_name: str) -> ndarray:
    ...



from collections import defaultdict
from functools import partial
from itertools import combinations

from ..classes.graph import Graph
from ..utils import arbitrary_element
from ..utils.decorators import not_implemented_for

__all__ = ["line_graph", "inverse_line_graph"]

def line_graph(G: Graph, create_using=None): ...
def inverse_line_graph(G: Graph): ...



from .boxplot import (
    boxplot as boxplot,
    boxplot_frame as boxplot_frame,
    boxplot_frame_groupby as boxplot_frame_groupby,
)
from .converter import (
    deregister as deregister,
    register as register,
)
from .hist import (
    hist_frame as hist_frame,
    hist_series as hist_series,
)
from .misc import (
    andrews_curves as andrews_curves,
    autocorrelation_plot as autocorrelation_plot,
    bootstrap_plot as bootstrap_plot,
    lag_plot as lag_plot,
    parallel_coordinates as parallel_coordinates,
    radviz as radviz,
    scatter_matrix as scatter_matrix,
)
from .tools import table as table

def plot(data, kind, **kwargs): ...



from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Union

color_names: Any
foreground: Any
background: Any
RESET: str
opt_dict: Any

def colorize(
    text: Optional[str] = ..., opts: Sequence[str] = ..., **kwargs: Any
) -> str: ...
def make_style(opts: Tuple[Any, ...] = ..., **kwargs: Any) -> Callable[..., Any]: ...

NOCOLOR_PALETTE: str
DARK_PALETTE: str
LIGHT_PALETTE: str
PALETTES: Any
DEFAULT_PALETTE: str = ...

def parse_color_setting(
    config_string: str,
) -> Optional[Dict[str, Dict[str, Union[Tuple[str], str]]]]: ...



from typing import Any, Dict, Iterable, List, Optional

from django.template.base import Origin, Template
from django.template.engine import Engine

class Loader:
    engine: Engine = ...
    get_template_cache: Dict[str, Any] = ...
    def __init__(self, engine: Engine) -> None: ...
    def get_template(
        self, template_name: str, skip: Optional[List[Origin]] = ...
    ) -> Template: ...
    def get_template_sources(self, template_name: str) -> Iterable[Origin]: ...
    def reset(self) -> None: ...



from typing import Mapping
from numpy.typing import ArrayLike
from ..classes.graph import Graph
from ..utils import np_random_state

__all__ = [
    "bipartite_layout",
    "circular_layout",
    "kamada_kawai_layout",
    "random_layout",
    "rescale_layout",
    "rescale_layout_dict",
    "shell_layout",
    "spring_layout",
    "spectral_layout",
    "planar_layout",
    "fruchterman_reingold_layout",
    "spiral_layout",
    "multipartite_layout",
]

@np_random_state(3)
def random_layout(
    G: Graph, center: ArrayLike | None = None, dim: int = 2, seed=None
) -> Mapping: ...
def circular_layout(
    G: Graph, scale=1, center: ArrayLike | None = None, dim: int = 2
) -> Mapping: ...
def shell_layout(
    G: Graph,
    nlist=None,
    rotate=None,
    scale=1,
    center: ArrayLike | None = None,
    dim: int = 2,
) -> Mapping: ...
def bipartite_layout(
    G: Graph,
    nodes,
    align="vertical",
    scale=1,
    center: ArrayLike | None = None,
    aspect_ratio=...,
) -> Mapping: ...
@np_random_state(10)
def spring_layout(
    G: Graph,
    k=None,
    pos=None,
    fixed=None,
    iterations=50,
    threshold=1e-4,
    weight="weight",
    scale=1,
    center: ArrayLike | None = None,
    dim: int = 2,
    seed=None,
) -> Mapping: ...

fruchterman_reingold_layout = ...

def kamada_kawai_layout(
    G: Graph,
    dist=None,
    pos=None,
    weight="weight",
    scale=1,
    center: ArrayLike | None = None,
    dim: int = 2,
) -> Mapping: ...
def spectral_layout(
    G: Graph, weight="weight", scale=1, center: ArrayLike | None = None, dim: int = 2
) -> Mapping: ...
def planar_layout(
    G: Graph, scale=1, center: ArrayLike | None = None, dim: int = 2
) -> Mapping: ...
def spiral_layout(
    G: Graph,
    scale=1,
    center: ArrayLike | None = None,
    dim: int = 2,
    resolution: float = 0.35,
    equidistant: bool = False,
) -> Mapping: ...
def multipartite_layout(
    G: Graph,
    subset_key="subset",
    align="vertical",
    scale=1,
    center: ArrayLike | None = None,
) -> Mapping: ...
def rescale_layout(pos, scale=1): ...
def rescale_layout_dict(pos, scale=1): ...



from ._rbm import BernoulliRBM as BernoulliRBM
from ._multilayer_perceptron import (
    MLPClassifier as MLPClassifier,
    MLPRegressor as MLPRegressor,
)

__all__ = ["BernoulliRBM", "MLPClassifier", "MLPRegressor"]



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .logs import (
    logger as logger,
    set_log_level as set_log_level,
    use_log_level as use_log_level,
)  # noqa
from .config import (
    config as config,
    sys_info as sys_info,
    save_config as save_config,
    get_config_keys as get_config_keys,  # noqa
    set_data_dir as set_data_dir,
    _TempDir as _TempDir,
)  # noqa
from .fetching import load_data_file as load_data_file  # noqa
from .frozen import Frozen as Frozen  # noqa
from . import fonts as fonts  # noqa
from . import transforms as transforms  # noqa
from .wrappers import use as use, run_subprocess as run_subprocess  # noqa
from .bunch import SimpleBunch as SimpleBunch  # noqa



import numpy as np

def arrays_to_mgr(arrays, arr_names, index, columns, dtype=...): ...
def masked_rec_array_to_mgr(data, index, columns, dtype, copy): ...
def init_ndarray(values, index, columns, dtype=..., copy: bool = ...): ...
def init_dict(data, index, columns, dtype=...): ...
def prep_ndarray(values, copy=...) -> np.ndarray: ...
def extract_index(data): ...
def reorder_arrays(arrays, arr_columns, columns): ...
def get_names_from_index(data): ...
def to_arrays(data, columns, coerce_float: bool = ..., dtype=...): ...
def sanitize_index(data, index, copy: bool = ...): ...



from matplotlib.axes import Axes
from matplotlib.contour import ContourSet

class TriContourSet(ContourSet):
    def __init__(self, ax: Axes, *args, **kwargs) -> None: ...

def tricontour(ax: Axes, *args, **kwargs) -> TriContourSet: ...
def tricontourf(ax: Axes, *args, **kwargs) -> TriContourSet: ...



from typing import Callable, Generator, Iterable, Iterator

from matplotlib.backend_bases import NonGuiException
from .figure import Figure

from itertools import count
from typing import Callable, List

import abc
import contextlib

subprocess_creation_flags = ...

def adjusted_figsize(w: float, h: float, dpi: float, n: int) -> tuple[float, float]: ...

class MovieWriterRegistry:
    def __init__(self) -> None: ...
    def register(self, name: str) -> Callable: ...
    def is_available(self, name: str) -> bool: ...
    def __iter__(self)-> Generator: ...
    def list(self) -> list[MovieWriter]: ...
    def __getitem__(self, name) -> MovieWriter: ...

writers: MovieWriterRegistry = ...

class AbstractMovieWriter(abc.ABC):
    def __init__(
        self,
        fps: float = 5,
        metadata: dict[str, str]|None = None,
        codec=None,
        bitrate=None,
    ) -> None: ...
    @abc.abstractmethod
    def setup(self, fig: Figure, outfile: str, dpi: float|None = None)-> None: ...
    @property
    def frame_size(self) -> tuple[int, int]: ...
    @abc.abstractmethod
    def grab_frame(self, **savefig_kwargs)-> None: ...
    @abc.abstractmethod
    def finish(self)-> None: ...
    @contextlib.contextmanager
    def saving(self, fig: Figure, outfile, dpi: float, *args, **kwargs)->Generator[AbstractMovieWriter, None, None]: ...

class MovieWriter(AbstractMovieWriter):

    frame_format: str
    fig: Figure

    supported_formats = ...
    def __init__(
        self,
        fps: int = 5,
        codec: str | None = None,
        bitrate: int|None = None,
        extra_args: list[str] | None = None,
        metadata: dict[str, str]|None = None,
    ) -> None: ...
    def setup(self, fig: Figure, outfile: str, dpi: float|None = None)-> None: ...
    def finish(self)-> None: ...
    def grab_frame(self, **savefig_kwargs)-> None: ...
    @classmethod
    def bin_path(cls) -> str: ...
    @classmethod
    def isAvailable(cls) -> bool: ...

class FileMovieWriter(MovieWriter):
    def __init__(self, *args, **kwargs) -> None: ...
    def setup(
        self, fig: Figure, outfile: str, dpi: float|None = None, frame_prefix: str|None = None
    ): ...
    def __del__(self)-> None: ...
    @property
    def frame_format(self)-> str: ...
    @frame_format.setter
    def frame_format(self, frame_format: str)-> None: ...
    def grab_frame(self, **savefig_kwargs)-> None: ...
    def finish(self)-> None: ...

class PillowWriter(AbstractMovieWriter):
    @classmethod
    def isAvailable(cls)-> bool: ...
    def setup(self, fig: Figure, outfile: str, dpi: float|None = None)-> None: ...
    def grab_frame(self, **savefig_kwargs)-> None: ...
    def finish(self)-> None: ...

class FFMpegBase:
    @property
    def output_args(self) -> list[str]: ...

class FFMpegWriter(FFMpegBase, MovieWriter): ...

class FFMpegFileWriter(FFMpegBase, FileMovieWriter):

    supported_formats: list[str] = ...

class ImageMagickBase:
    @classmethod
    def bin_path(cls)-> str: ...
    @classmethod
    def isAvailable(cls)-> bool: ...

class ImageMagickWriter(ImageMagickBase, MovieWriter):

    input_names: str = ...

class ImageMagickFileWriter(ImageMagickBase, FileMovieWriter):

    supported_formats: list[str] = ...
    input_names: str = ...

class HTMLWriter(FileMovieWriter):

    supported_formats: list[str] = ...
    @classmethod
    def isAvailable(cls): ...
    def __init__(
        self,
        fps: float=30,
        codec=None,
        bitrate: float|None=None,
        extra_args=None,
        metadata=None,
        embed_frames: bool=False,
        default_mode: str='loop',
        embed_limit=None,
    ) -> None: ...
    def setup(self, fig: Figure, outfile: str, dpi: float, frame_dir=None)-> None: ...
    def grab_frame(self, **savefig_kwargs)-> None: ...
    def finish(self)-> None: ...

class Animation:
    def __init__(
        self, fig: Figure, event_source: object = None, blit: bool = False
    ) -> None: ...
    def __del__(self)-> None: ...
    def save(
        self,
        filename: str,
        writer: MovieWriter | str|None = None,
        fps: int|None = None,
        dpi: float|None = None,
        codec: str|None = None,
        bitrate: int|None = None,
        extra_args: list[str] | None = None,
        metadata: dict[str, str]|None = None,
        extra_anim: list|None = None,
        savefig_kwargs: dict|None = None,
        *,
        progress_callback: Callable|None = None
    )-> None: ...
    def new_frame_seq(self) -> Iterator: ...
    def new_saved_frame_seq(self)-> Iterable: ...
    def to_html5_video(self, embed_limit: float|None = None) -> str: ...
    def to_jshtml(
        self, fps: int|None = None, embed_frames: bool = True, default_mode: str|None = None
    ): ...
    def pause(self)-> None: ...
    def resume(self)-> None: ...

class TimedAnimation(Animation):
    def __init__(
        self,
        fig: Figure,
        interval: int = 200,
        repeat_delay: int = 0,
        repeat: bool = True,
        event_source=None,
        *args,
        **kwargs
    ) -> None: ...

class ArtistAnimation(TimedAnimation):
    def __init__(self, fig: Figure, artists: list, *args, **kwargs) -> None: ...

class FuncAnimation(TimedAnimation):
    def __init__(
        self,
        fig: Figure,
        func: Callable,
        frames: Callable | Iterable | int | None = None,
        init_func: Callable|None = None,
        fargs: tuple | None =None,
        save_count: int|None = None,
        *,
        cache_frame_data: bool = True,
        **kwargs
    ) -> None: ...
    def new_frame_seq(self) -> count: ...
    def new_saved_frame_seq(self)-> Iterator: ...



from typing import Mapping

from ...classes.graph import Graph

__all__ = ["degree_centrality", "betweenness_centrality", "closeness_centrality"]

def degree_centrality(G: Graph, nodes) -> Mapping: ...
def betweenness_centrality(G: Graph, nodes) -> Mapping: ...
def closeness_centrality(G: Graph, nodes, normalized: bool = True) -> Mapping: ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from ..exceptions import DataConversionWarning as DataConversionWarning
from numpy import ndarray
from ..utils._param_validation import (
    Interval as Interval,
    HasMethods as HasMethods,
    StrOptions as StrOptions,
)
from numbers import Integral as Integral, Real as Real
from joblib import Memory
from ..neighbors import NearestNeighbors as NearestNeighbors
from ..metrics import pairwise_distances as pairwise_distances
from ..base import BaseEstimator, ClusterMixin
from scipy.sparse import (
    issparse as issparse,
    SparseEfficiencyWarning as SparseEfficiencyWarning,
)
from ..metrics.pairwise import PAIRWISE_BOOLEAN_FUNCTIONS as PAIRWISE_BOOLEAN_FUNCTIONS
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..utils import gen_batches as gen_batches, get_chunk_n_rows as get_chunk_n_rows
from ..utils.validation import check_memory as check_memory

OPTICS_Self = TypeVar("OPTICS_Self", bound="OPTICS")


import warnings
import numpy as np


class OPTICS(ClusterMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    cluster_hierarchy_: ndarray = ...
    predecessor_: ndarray = ...
    core_distances_: ndarray = ...
    ordering_: ndarray = ...
    reachability_: ndarray = ...
    labels_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        min_samples: float | int = 5,
        max_eps: Float = ...,
        metric: str | Callable = "minkowski",
        p: Float = 2,
        metric_params: None | dict = None,
        cluster_method: str = "xi",
        eps: None | Float = None,
        xi: float = 0.05,
        predecessor_correction: bool = True,
        min_cluster_size: float | None | int = None,
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        memory: None | Memory | str = None,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(self: OPTICS_Self, X: MatrixLike, y: Any = None) -> OPTICS_Self:
        ...


def compute_optics_graph(
    X: MatrixLike,
    *,
    min_samples: float | int,
    max_eps: Float,
    metric: str | Callable,
    p: Int,
    metric_params: dict,
    algorithm: Literal["auto", "ball_tree", "kd_tree", "brute"],
    leaf_size: Int,
    n_jobs: Int,
) -> tuple[ndarray, ndarray, ndarray, ndarray]:
    ...


def cluster_optics_dbscan(
    *,
    reachability: ArrayLike,
    core_distances: ArrayLike,
    ordering: ArrayLike,
    eps: Float,
) -> ndarray:
    ...


def cluster_optics_xi(
    *,
    reachability: ArrayLike,
    predecessor: ArrayLike,
    ordering: ArrayLike,
    min_samples: float | int,
    min_cluster_size: float | None | int = None,
    xi: float = 0.05,
    predecessor_correction: bool = True,
) -> tuple[ndarray, ndarray]:
    ...



import numpy as np
import scipy.ndimage as ndi

def _marching_cubes_classic(volume, level, spacing, gradient_direction): ...
def mesh_surface_area(verts, faces) -> float: ...
def _correct_mesh_orientation(
    volume, actual_verts, faces, spacing=(1.0, 1.0, 1.0), gradient_direction="descent"
): ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from itertools import product
from string import hexdigits

# See <http://www.w3.org/TR/SVG/types.html#ColorKeywords>
_keyword_colors: dict = ...

_HEXDEC: dict = ...

def _rgb(triplet): ...

class Color(object):
    def __init__(self, content): ...
    @property
    def rgb(self): ...
    @property
    def rgba(self): ...
    def __repr__(self): ...



from ._label_propagation import (
    LabelPropagation as LabelPropagation,
    LabelSpreading as LabelSpreading,
)
from ._self_training import SelfTrainingClassifier as SelfTrainingClassifier

__all__ = ["SelfTrainingClassifier", "LabelPropagation", "LabelSpreading"]



from math import sqrt

from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = ["is_threshold_graph", "find_threshold_graph"]

def is_threshold_graph(G: Graph) -> bool: ...
def is_threshold_sequence(degree_sequence): ...
def creation_sequence(degree_sequence, with_labels=False, compact=False): ...
def make_compact(creation_sequence): ...
def uncompact(creation_sequence): ...
def creation_sequence_to_weights(creation_sequence): ...
def weights_to_creation_sequence(
    weights, threshold=1, with_labels=False, compact=False
): ...

# Manipulating NetworkX.Graphs in context of threshold graphs
def threshold_graph(creation_sequence, create_using=None): ...
def find_alternating_4_cycle(G: Graph): ...
def find_threshold_graph(G: Graph, create_using=None): ...
def find_creation_sequence(G: Graph): ...

# Properties of Threshold Graphs
def triangles(creation_sequence): ...
def triangle_sequence(creation_sequence): ...
def cluster_sequence(creation_sequence): ...
def degree_sequence(creation_sequence): ...
def density(creation_sequence): ...
def degree_correlation(creation_sequence): ...
def shortest_path(creation_sequence, u, v): ...
def shortest_path_length(creation_sequence, i): ...
def betweenness_sequence(creation_sequence, normalized=True): ...
def eigenvectors(creation_sequence): ...
def spectral_projection(u, eigenpairs): ...
def eigenvalues(creation_sequence): ...

# Threshold graph creation routines

@py_random_state(2)
def random_threshold_sequence(n, p, seed=None): ...

# maybe *_d_threshold_sequence routines should
# be (or be called from) a single routine with a more descriptive name
# and a keyword parameter?
def right_d_threshold_sequence(n, m): ...
def left_d_threshold_sequence(n, m): ...
@py_random_state(3)
def swap_d(cs, p_split=1.0, p_combine=1.0, seed=None): ...



from typing import Any, Callable, Literal, Mapping, Sequence, Type
from abc import ABC, abstractmethod
from numbers import Integral, Real
from scipy.sparse import issparse as issparse, csr_matrix as csr_matrix
from inspect import signature as signature
from .._typing import Float
from collections.abc import Iterable as Iterable
import functools
import math
import operator
import re
import warnings

import numpy as np


class InvalidParameterError(ValueError, TypeError):
    ...


def validate_parameter_constraints(
    parameter_constraints: Mapping | str, params: dict, caller_name: str
) -> None:
    ...


def make_constraint(constraint: Any) -> _Constraint:
    ...


def validate_params(parameter_constraints: dict) -> Callable:
    ...


class _Constraint(ABC):
    def __init__(self) -> None:
        ...

    @abstractmethod
    def is_satisfied_by(self, val: Any) -> bool:
        ...

    @abstractmethod
    def __str__(self) -> str:
        ...


class _InstancesOf(_Constraint):
    def __init__(self, type) -> None:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _NoneConstraint(_Constraint):
    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _NanConstraint(_Constraint):
    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _PandasNAConstraint(_Constraint):
    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class Options(_Constraint):
    def __init__(
        self,
        type: Type[str] | Type[type] | Type[Real],
        options: set | set[float] | set[str] | set[Type[Float] | Type[Float]],
        *,
        deprecated: set | None = None
    ) -> None:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class StrOptions(Options):
    def __init__(
        self, options: set[str], *, deprecated: set[str] | None = None
    ) -> None:
        ...


class Interval(_Constraint):
    def __init__(
        self,
        type: Real | Type[Integral] | Type[Real] | Integral,
        left: float | None | int,
        right: float | None | int,
        *,
        closed: Literal["left", "right", "both", "neither"]
    ) -> None:
        ...

    def __contains__(self, val: bool | Float) -> bool:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _ArrayLikes(_Constraint):
    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _SparseMatrices(_Constraint):
    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _Callables(_Constraint):
    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _RandomStates(_Constraint):
    def __init__(self) -> None:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _Booleans(_Constraint):
    def __init__(self) -> None:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _VerboseHelper(_Constraint):
    def __init__(self) -> None:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _MissingValues(_Constraint):
    def __init__(self) -> None:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class HasMethods(_Constraint):
    def __init__(self, methods: list[str] | str | Sequence[str]) -> None:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _IterablesNotString(_Constraint):
    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class _CVObjects(_Constraint):
    def __init__(self) -> None:
        ...

    def is_satisfied_by(self, val: Any) -> bool:
        ...

    def __str__(self) -> str:
        ...


class Hidden:
    def __init__(self, constraint: _Constraint | str) -> None:
        ...


def generate_invalid_param_val(
    constraint: _Constraint, constraints: None | list[_Constraint] = None
) -> Any:
    ...


def generate_valid_param(constraint: _Constraint) -> Any:
    ...



"""
This type stub file was generated by pyright.
"""

from traitlets.config.configurable import Configurable
from traitlets import Integer

"""
Prefiltering components.

Prefilters transform user input before it is exec'd by Python.  These
transforms are used to implement additional syntax such as !ls and %magic.
"""
class PrefilterError(Exception):
    ...


re_fun_name = ...
re_exclude_auto = ...
def is_shadowed(identifier, ip):
    """Is the given identifier defined in one of the namespaces which shadow
    the alias and magic namespaces?  Note that an identifier is different
    than ifun, because it can not contain a '.' character."""
    ...

class PrefilterManager(Configurable):
    """Main prefilter component.

    The IPython prefilter is run on all user input before it is run.  The
    prefilter consumes lines of input and produces transformed lines of
    input.

    The implementation consists of two phases:

    1. Transformers
    2. Checkers and handlers

    Over time, we plan on deprecating the checkers and handlers and doing
    everything in the transformers.

    The transformers are instances of :class:`PrefilterTransformer` and have
    a single method :meth:`transform` that takes a line and returns a
    transformed line.  The transformation can be accomplished using any
    tool, but our current ones use regular expressions for speed.

    After all the transformers have been run, the line is fed to the checkers,
    which are instances of :class:`PrefilterChecker`.  The line is passed to
    the :meth:`check` method, which either returns `None` or a
    :class:`PrefilterHandler` instance.  If `None` is returned, the other
    checkers are tried.  If an :class:`PrefilterHandler` instance is returned,
    the line is passed to the :meth:`handle` method of the returned
    handler and no further checkers are tried.

    Both transformers and checkers have a `priority` attribute, that determines
    the order in which they are called.  Smaller priorities are tried first.

    Both transformers and checkers also have `enabled` attribute, which is
    a boolean that determines if the instance is used.

    Users or developers can change the priority or enabled attribute of
    transformers or checkers, but they must call the :meth:`sort_checkers`
    or :meth:`sort_transformers` method after changing the priority.
    """
    multi_line_specials = ...
    shell = ...
    def __init__(self, shell=..., **kwargs) -> None:
        ...
    
    def sort_transformers(self): # -> None:
        """Sort the transformers by priority.

        This must be called after the priority of a transformer is changed.
        The :meth:`register_transformer` method calls this automatically.
        """
        ...
    
    @property
    def transformers(self):
        """Return a list of checkers, sorted by priority."""
        ...
    
    def register_transformer(self, transformer): # -> None:
        """Register a transformer instance."""
        ...
    
    def unregister_transformer(self, transformer): # -> None:
        """Unregister a transformer instance."""
        ...
    
    def init_checkers(self): # -> None:
        """Create the default checkers."""
        ...
    
    def sort_checkers(self): # -> None:
        """Sort the checkers by priority.

        This must be called after the priority of a checker is changed.
        The :meth:`register_checker` method calls this automatically.
        """
        ...
    
    @property
    def checkers(self):
        """Return a list of checkers, sorted by priority."""
        ...
    
    def register_checker(self, checker): # -> None:
        """Register a checker instance."""
        ...
    
    def unregister_checker(self, checker): # -> None:
        """Unregister a checker instance."""
        ...
    
    def init_handlers(self): # -> None:
        """Create the default handlers."""
        ...
    
    @property
    def handlers(self):
        """Return a dict of all the handlers."""
        ...
    
    def register_handler(self, name, handler, esc_strings): # -> None:
        """Register a handler instance by name with esc_strings."""
        ...
    
    def unregister_handler(self, name, handler, esc_strings): # -> None:
        """Unregister a handler instance by name with esc_strings."""
        ...
    
    def get_handler_by_name(self, name):
        """Get a handler by its name."""
        ...
    
    def get_handler_by_esc(self, esc_str):
        """Get a handler by its escape string."""
        ...
    
    def prefilter_line_info(self, line_info):
        """Prefilter a line that has been converted to a LineInfo object.

        This implements the checker/handler part of the prefilter pipe.
        """
        ...
    
    def find_handler(self, line_info):
        """Find a handler for the line_info by trying checkers."""
        ...
    
    def transform_line(self, line, continue_prompt):
        """Calls the enabled transformers in order of increasing priority."""
        ...
    
    def prefilter_line(self, line, continue_prompt=...):
        """Prefilter a single input line as text.

        This method prefilters a single line of text by calling the
        transformers and then the checkers/handlers.
        """
        ...
    
    def prefilter_lines(self, lines, continue_prompt=...):
        """Prefilter multiple input lines of text.

        This is the main entry point for prefiltering multiple lines of
        input.  This simply calls :meth:`prefilter_line` for each line of
        input.

        This covers cases where there are multiple lines in the user entry,
        which is the case when the user goes back to a multiline history
        entry and presses enter.
        """
        ...
    


class PrefilterTransformer(Configurable):
    """Transform a line of user input."""
    priority = Integer(100).tag(config=True)
    shell = ...
    prefilter_manager = ...
    enabled = ...
    def __init__(self, shell=..., prefilter_manager=..., **kwargs) -> None:
        ...
    
    def transform(self, line, continue_prompt): # -> None:
        """Transform a line, returning the new one."""
        ...
    
    def __repr__(self):
        ...
    


class PrefilterChecker(Configurable):
    """Inspect an input line and return a handler for that line."""
    priority = Integer(100).tag(config=True)
    shell = ...
    prefilter_manager = ...
    enabled = ...
    def __init__(self, shell=..., prefilter_manager=..., **kwargs) -> None:
        ...
    
    def check(self, line_info): # -> None:
        """Inspect line_info and return a handler instance or None."""
        ...
    
    def __repr__(self):
        ...
    


class EmacsChecker(PrefilterChecker):
    priority = Integer(100).tag(config=True)
    enabled = ...
    def check(self, line_info): # -> None:
        "Emacs ipython-mode tags certain input lines."
        ...
    


class MacroChecker(PrefilterChecker):
    priority = Integer(250).tag(config=True)
    def check(self, line_info): # -> None:
        ...
    


class IPyAutocallChecker(PrefilterChecker):
    priority = Integer(300).tag(config=True)
    def check(self, line_info): # -> None:
        "Instances of IPyAutocall in user_ns get autocalled immediately"
        ...
    


class AssignmentChecker(PrefilterChecker):
    priority = Integer(600).tag(config=True)
    def check(self, line_info): # -> None:
        """Check to see if user is assigning to a var for the first time, in
        which case we want to avoid any sort of automagic / autocall games.

        This allows users to assign to either alias or magic names true python
        variables (the magic/alias systems always take second seat to true
        python code).  E.g. ls='hi', or ls,that=1,2"""
        ...
    


class AutoMagicChecker(PrefilterChecker):
    priority = Integer(700).tag(config=True)
    def check(self, line_info): # -> None:
        """If the ifun is magic, and automagic is on, run it.  Note: normal,
        non-auto magic would already have been triggered via '%' in
        check_esc_chars. This just checks for automagic.  Also, before
        triggering the magic handler, make sure that there is nothing in the
        user namespace which could shadow it."""
        ...
    


class PythonOpsChecker(PrefilterChecker):
    priority = Integer(900).tag(config=True)
    def check(self, line_info): # -> None:
        """If the 'rest' of the line begins with a function call or pretty much
        any python operator, we should simply execute the line (regardless of
        whether or not there's a possible autocall expansion).  This avoids
        spurious (and very confusing) geattr() accesses."""
        ...
    


class AutocallChecker(PrefilterChecker):
    priority = Integer(1000).tag(config=True)
    function_name_regexp = ...
    exclude_regexp = ...
    def check(self, line_info): # -> None:
        "Check if the initial word/function is callable and autocall is on."
        ...
    


class PrefilterHandler(Configurable):
    handler_name = ...
    esc_strings = ...
    shell = ...
    prefilter_manager = ...
    def __init__(self, shell=..., prefilter_manager=..., **kwargs) -> None:
        ...
    
    def handle(self, line_info):
        """Handle normal input lines. Use as a template for handlers."""
        ...
    
    def __str__(self) -> str:
        ...
    


class MacroHandler(PrefilterHandler):
    handler_name = ...
    def handle(self, line_info):
        ...
    


class MagicHandler(PrefilterHandler):
    handler_name = ...
    esc_strings = ...
    def handle(self, line_info):
        """Execute magic functions."""
        ...
    


class AutoHandler(PrefilterHandler):
    handler_name = ...
    esc_strings = ...
    def handle(self, line_info):
        """Handle lines which can be auto-executed, quoting if requested."""
        ...
    


class EmacsHandler(PrefilterHandler):
    handler_name = ...
    esc_strings = ...
    def handle(self, line_info):
        """Handle input lines marked by python-mode."""
        ...
    


_default_checkers = ...
_default_handlers = ...



import numpy as np
from scipy.spatial.distance import cdist

def match_descriptors(
    descriptors1,
    descriptors2,
    metric=None,
    p: int = 2,
    max_distance: float = ...,
    cross_check: bool = True,
    max_ratio: float = 1.0,
): ...



from numpy import ndarray
from vispy.visuals.transforms.linear import STTransform
from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from ...util import transforms
from ...geometry import Rect
from ._util import arg_to_vec4, as_vec4
from .base_transform import BaseTransform

class NullTransform(BaseTransform):

    glsl_map: str = ...
    glsl_imap: str = ...

    Linear: bool = ...
    Orthogonal: bool = ...
    NonScaling: bool = ...
    Isometric: bool = ...

    @arg_to_vec4
    def map(self, coords: ArrayLike): ...
    def imap(self, coords: ArrayLike): ...
    def __mul__(self, tr): ...
    def __rmul__(self, tr): ...

class STTransform(BaseTransform):

    glsl_map: str = ...

    glsl_imap: str = ...

    Linear: bool = ...
    Orthogonal: bool = ...
    NonScaling: bool = ...
    Isometric: bool = ...

    def __init__(self, scale: ArrayLike | None = None, translate: ArrayLike | None = None): ...
    @arg_to_vec4
    def map(self, coords: ArrayLike) -> NDArray: ...
    @arg_to_vec4
    def imap(self, coords: ArrayLike) -> NDArray: ...
    def shader_map(self): ...
    def shader_imap(self): ...
    @property
    def scale(self): ...
    @scale.setter
    def scale(self, s): ...
    @property
    def translate(self): ...
    @translate.setter
    def translate(self, t): ...
    def _set_st(self, scale=None, translate=None, update=True): ...
    def _update_shaders(self): ...
    def move(self, move: ArrayLike): ...
    def zoom(self, zoom: ArrayLike, center: ArrayLike = ..., mapped: bool = True): ...
    def as_matrix(self): ...
    @classmethod
    def from_mapping(cls, x0: ArrayLike, x1: ArrayLike) -> STTransform: ...
    def set_mapping(self, x0: ArrayLike, x1: ArrayLike, update: bool = True): ...
    def __mul__(self, tr): ...
    def __rmul__(self, tr): ...
    def __repr__(self): ...

class MatrixTransform(BaseTransform):

    glsl_map: str = ...

    glsl_imap: str = ...

    Linear: bool = ...
    Orthogonal: bool = ...
    NonScaling: bool = ...
    Isometric: bool = ...

    def __init__(self, matrix: ArrayLike | None = None): ...
    @arg_to_vec4
    def map(self, coords: ArrayLike) -> NDArray: ...
    @arg_to_vec4
    def imap(self, coords: ArrayLike) -> NDArray: ...
    def shader_map(self): ...
    def shader_imap(self): ...
    @property
    def matrix(self): ...
    @matrix.setter
    def matrix(self, m): ...
    @property
    def inv_matrix(self): ...
    @arg_to_vec4
    def translate(self, pos: np.ndarray): ...
    def scale(self, scale: ArrayLike, center: ArrayLike | None = None): ...
    def rotate(self, angle: float, axis: ArrayLike): ...
    def set_mapping(self, points1: ArrayLike, points2: ArrayLike): ...
    def set_ortho(self, l: float, r: float, b: float, t: float, n: float, f: float): ...
    def reset(self): ...
    def __mul__(self, tr): ...
    def __repr__(self): ...
    def set_perspective(self, fov: float, aspect: float, near: float, far: float): ...
    def set_frustum(self, l: float, r: float, b: float, t: float, n: float, f: float): ...

# class SRTTransform(BaseTransform):
#    """ Transform performing scale, rotate, and translate, in that order.
#
#    This transformation allows objects to be placed arbitrarily in a scene
#    much the same way MatrixTransform does. However, an incorrect order of
#    operations in MatrixTransform may result in shearing the object (if scale
#    is applied after rotate) or in unpredictable translation (if scale/rotate
#    is applied after translation). SRTTransform avoids these problems by
#    enforcing the correct order of operations.
#    """
#    # TODO



from typing import Any, Callable, ClassVar, Iterable, Literal, Mapping, TypeVar
from operator import itemgetter as itemgetter
from collections import defaultdict as defaultdict
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    check_array as check_array,
    FLOAT_DTYPES as FLOAT_DTYPES,
)
from ..exceptions import NotFittedError as NotFittedError
from numpy import ndarray
from ..utils._param_validation import (
    StrOptions as StrOptions,
    Interval as Interval,
    HasMethods as HasMethods,
)
from numbers import Integral as Integral, Real as Real
from functools import partial
from ._stop_words import ENGLISH_STOP_WORDS
from ..base import BaseEstimator, TransformerMixin, OneToOneFeatureMixin
from scipy.sparse import spmatrix
from ._hash import FeatureHasher as FeatureHasher
from .._typing import ArrayLike, Int, MatrixLike, Float
from collections.abc import Mapping
from ..preprocessing import normalize as normalize

TfidfVectorizer_Self = TypeVar("TfidfVectorizer_Self", bound="TfidfVectorizer")
HashingVectorizer_Self = TypeVar("HashingVectorizer_Self", bound="HashingVectorizer")
CountVectorizer_Self = TypeVar("CountVectorizer_Self", bound="CountVectorizer")
TfidfTransformer_Self = TypeVar("TfidfTransformer_Self", bound="TfidfTransformer")

# Authors: Olivier Grisel <olivier.grisel@ensta.org>
#          Mathieu Blondel <mathieu@mblondel.org>
#          Lars Buitinck
#          Robert Layton <robertlayton@gmail.com>
#          Jochen Wersdörfer <jochen@wersdoerfer.de>
#          Roman Sinayev <roman.sinayev@gmail.com>
#
# License: BSD 3 clause

import array
import re
import unicodedata
import warnings

import numpy as np
import scipy.sparse as sp


__all__ = [
    "HashingVectorizer",
    "CountVectorizer",
    "ENGLISH_STOP_WORDS",
    "TfidfTransformer",
    "TfidfVectorizer",
    "strip_accents_ascii",
    "strip_accents_unicode",
    "strip_tags",
]


def strip_accents_unicode(s: str) -> str:
    ...


def strip_accents_ascii(s: str) -> str:
    ...


def strip_tags(s: str) -> str:
    ...


class _VectorizerMixin:

    _white_spaces = ...

    def decode(self, doc: bytes | str) -> str:
        ...

    def build_preprocessor(self) -> Callable | partial:
        ...

    def build_tokenizer(self) -> Callable:
        ...

    def get_stop_words(self) -> frozenset | None | list:
        ...

    def build_analyzer(self) -> Callable | partial:
        ...


class HashingVectorizer(
    TransformerMixin, _VectorizerMixin, BaseEstimator, auto_wrap_output_keys=None
):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        input: Literal["filename", "file", "content", "content"] = "content",
        encoding: str = "utf-8",
        decode_error: Literal["strict", "ignore", "replace", "strict"] = "strict",
        strip_accents: None | Literal["ascii", "unicode"] | Callable = None,
        lowercase: bool = True,
        preprocessor: None | Callable = None,
        tokenizer: None | Callable = None,
        stop_words: None | str | ArrayLike = None,
        token_pattern: None | str = r"(?u)\b\w\w+\b",
        ngram_range: tuple[float, float] = ...,
        analyzer: Literal["word", "char", "char_wb", "word"] | Callable = "word",
        n_features: Int = ...,
        binary: bool = False,
        norm: Literal["l1", "l2", "l2"] = "l2",
        alternate_sign: bool = True,
        dtype=...,
    ) -> None:
        ...

    def partial_fit(
        self: HashingVectorizer_Self, X: MatrixLike, y: Any = None
    ) -> HashingVectorizer_Self:
        ...

    def fit(
        self: HashingVectorizer_Self, X: list[str] | MatrixLike, y: Any = None
    ) -> HashingVectorizer_Self:
        ...

    def transform(self, X: Iterable[str]) -> spmatrix:
        ...

    def fit_transform(self, X: list[str] | Iterable[str], y: Any = None) -> spmatrix:
        ...


class CountVectorizer(_VectorizerMixin, BaseEstimator):
    stop_words_: set = ...
    fixed_vocabulary_: bool = ...
    vocabulary_: dict = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        input: Literal["filename", "file", "content", "content"] = "content",
        encoding: str = "utf-8",
        decode_error: Literal["strict", "ignore", "replace", "strict"] = "strict",
        strip_accents: None | Literal["ascii", "unicode"] | Callable = None,
        lowercase: bool = True,
        preprocessor: None | Callable = None,
        tokenizer: None | Callable = None,
        stop_words: None | str | ArrayLike = None,
        token_pattern: None | str = r"(?u)\b\w\w+\b",
        ngram_range: tuple[float, float] = ...,
        analyzer: Literal["word", "char", "char_wb", "word"] | Callable = "word",
        max_df: float | int = 1.0,
        min_df: float | int = 1,
        max_features: None | Int = None,
        vocabulary: Iterable | None | Mapping = None,
        binary: bool = False,
        dtype=...,
    ) -> None:
        ...

    def fit(
        self: CountVectorizer_Self, raw_documents: Iterable, y=None
    ) -> CountVectorizer_Self:
        ...

    def fit_transform(
        self,
        raw_documents: list[str] | ndarray | Iterable,
        y: None | ndarray | list[Int] = None,
    ) -> ndarray | spmatrix:
        ...

    def transform(self, raw_documents: list[str] | ndarray | Iterable) -> spmatrix:
        ...

    def inverse_transform(self, X: MatrixLike | ArrayLike) -> list[ndarray]:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...


class TfidfTransformer(
    OneToOneFeatureMixin, TransformerMixin, BaseEstimator, auto_wrap_output_keys=None
):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        norm: None | Literal["l1", "l2", "l2"] = "l2",
        use_idf: bool = True,
        smooth_idf: bool = True,
        sublinear_tf: bool = False,
    ) -> None:
        ...

    def fit(
        self: TfidfTransformer_Self, X: MatrixLike, y: None | ndarray | list[Int] = None
    ) -> TfidfTransformer_Self:
        ...

    def transform(self, X: MatrixLike, copy: bool = True) -> spmatrix:
        ...

    @property
    def idf_(self) -> ndarray:
        ...

    @idf_.setter
    def idf_(self, value) -> ndarray:
        ...


class TfidfVectorizer(CountVectorizer):
    stop_words_: set = ...
    fixed_vocabulary_: bool = ...
    vocabulary_: dict = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        input: Literal["filename", "file", "content", "content"] = "content",
        encoding: str = "utf-8",
        decode_error: Literal["strict", "ignore", "replace", "strict"] = "strict",
        strip_accents: None | Literal["ascii", "unicode"] | Callable = None,
        lowercase: bool = True,
        preprocessor: None | Callable = None,
        tokenizer: None | Callable = None,
        analyzer: Literal["word", "char", "char_wb", "word"] | Callable = "word",
        stop_words: None | str | ArrayLike = None,
        token_pattern: str = r"(?u)\b\w\w+\b",
        ngram_range: tuple[float, float] = ...,
        max_df: Float = 1.0,
        min_df: Float = 1,
        max_features: None | Int = None,
        vocabulary: Iterable | None | Mapping = None,
        binary: bool = False,
        dtype=...,
        norm: None | Literal["l1", "l2", "l2"] = "l2",
        use_idf: bool = True,
        smooth_idf: bool = True,
        sublinear_tf: bool = False,
    ) -> None:
        ...

    # Broadcast the TF-IDF parameters to the underlying transformer instance
    # for easy grid search and repr

    @property
    def idf_(self) -> ndarray:
        ...

    @idf_.setter
    def idf_(self, value) -> ndarray:
        ...

    def fit(
        self: TfidfVectorizer_Self, raw_documents: Iterable, y=None
    ) -> TfidfVectorizer_Self:
        ...

    def fit_transform(
        self, raw_documents: list[str] | ndarray | Iterable, y: None | ndarray = None
    ) -> spmatrix:
        ...

    def transform(self, raw_documents: list[str] | ndarray | Iterable) -> spmatrix:
        ...



from vispy.util.svg.transform import Transform
from typing import Sequence
from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ..shaders import FunctionChain
from .base_transform import BaseTransform
from .linear import NullTransform

class ChainTransform(BaseTransform):

    glsl_map = ...
    glsl_imap = ...

    def __init__(self, *transforms: Sequence[BaseTransform]): ...
    @property
    def transforms(self): ...
    @transforms.setter
    def transforms(self, tr): ...
    @property
    def simplified(self): ...
    @property
    def Linear(self): ...
    @property
    def Orthogonal(self): ...
    @property
    def NonScaling(self): ...
    @property
    def Isometric(self): ...
    def map(self, coords: ArrayLike) -> NDArray: ...
    def imap(self, coords: ArrayLike) -> NDArray: ...
    def shader_map(self): ...
    def shader_imap(self): ...
    def _rebuild_shaders(self): ...
    def append(self, tr: Transform): ...
    def prepend(self, tr: Transform): ...
    def _subtr_changed(self, ev): ...
    def __setitem__(self, index, tr): ...
    def __mul__(self, tr): ...
    def __rmul__(self, tr): ...
    def __str__(self): ...
    def __repr__(self): ...
    def __del__(self): ...

class SimplifiedChainTransform(ChainTransform):
    def __init__(self, chain): ...
    def source_changed(self, event): ...



"""
This type stub file was generated by pyright.
"""

"""Support for wildcard pattern matching in object inspection.

Authors
-------
- Jörgen Stenarson <jorgen.stenarson@bostream.nu>
- Thomas Kluyver
"""
def create_typestr2type_dicts(dont_include_in_type2typestr=...):
    """Return dictionaries mapping lower case typename (e.g. 'tuple') to type
    objects from the types package, and vice versa."""
    ...

def is_type(obj, typestr_or_type):
    """is_type(obj, typestr_or_type) verifies if obj is of a certain type. It
    can take strings or actual python types for the second argument, i.e.
    'tuple'<->TupleType. 'all' matches all types.

    TODO: Should be extended for choosing more than one type."""
    ...

def show_hidden(str, show_all=...):
    """Return true for strings starting with single _ if show_all is true."""
    ...

def dict_dir(obj):
    """Produce a dictionary of an object's attributes. Builds on dir2 by
    checking that a getattr() call actually succeeds."""
    ...

def filter_ns(ns, name_pattern=..., type_pattern=..., ignore_case=..., show_all=...):
    """Filter a namespace dictionary by name pattern and item type."""
    ...

def list_namespace(namespace, type_pattern, filter, ignore_case=..., show_all=...):
    """Return dictionary of all objects in a namespace dictionary that match
    type_pattern and filter."""
    ...




from typing import Callable, Type
from matplotlib import backend_tools
from matplotlib._api import classproperty
from matplotlib.backend_bases import FigureCanvasBase, ToolContainerBase
from gi.repository import Gtk
from ._backend_gtk import (
    TimerGTK as TimerGTK3,
    _BackendGTK,
    _FigureManagerGTK,
    _NavigationToolbar2GTK,
)

class __getattr__:
    @property
    def cursord(self)-> dict: ...

    icon_filename: Callable = ...
    window_icon: Callable = ...

class FigureCanvasGTK3(Gtk.DrawingArea, FigureCanvasBase):
    required_interactive_framework: str = ...
    _timer_cls: Type[TimerGTK3] = TimerGTK3
    manager_class: classproperty = ...
    event_mask: int = ...

    def __init__(self, figure=...) -> None: ...
    def destroy(self)-> None: ...
    def set_cursor(self, cursor)-> None: ...
    def scroll_event(self, widget, event)-> bool: ...
    def button_press_event(self, widget, event)-> bool: ...
    def button_release_event(self, widget, event)-> bool: ...
    def key_press_event(self, widget, event)-> bool: ...
    def key_release_event(self, widget, event)-> bool: ...
    def motion_notify_event(self, widget, event)-> bool: ...
    def leave_notify_event(self, widget, event)-> None: ...
    def enter_notify_event(self, widget, event)-> None: ...
    def size_allocate(self, widget, allocation)-> None: ...
    def configure_event(self, widget, event)-> bool: ...
    def on_draw_event(self, widget, ctx)-> None: ...
    def draw(self)-> None: ...
    def draw_idle(self)-> None: ...
    def flush_events(self)-> None: ...

class NavigationToolbar2GTK3(_NavigationToolbar2GTK, Gtk.Toolbar):
    def __init__(self, canvas, window=...) -> None: ...

    win: Callable = ...
    def save_figure(self, *args)-> None: ...

class ToolbarGTK3(ToolContainerBase, Gtk.Box):
    def __init__(self, toolmanager) -> None: ...
    def add_toolitem(self, name, group, position, image_file, description, toggle)-> None: ...
    def toggle_toolitem(self, name, toggled)-> None: ...
    def remove_toolitem(self, name)-> None: ...
    def set_message(self, s: str)-> None: ...

class SaveFigureGTK3(backend_tools.SaveFigureBase):
    def trigger(self, *args, **kwargs)-> None: ...

class SetCursorGTK3(backend_tools.SetCursorBase):
    def set_cursor(self, cursor)-> None: ...

class HelpGTK3(backend_tools.ToolHelpBase):
    def trigger(self, *args)-> None: ...

class ToolCopyToClipboardGTK3(backend_tools.ToolCopyToClipboardBase):
    def trigger(self, *args, **kwargs)-> None: ...

def error_msg_gtk(msg, parent=...)-> None: ...

Toolbar: Type[ToolbarGTK3] = ToolbarGTK3

class FigureManagerGTK3(_FigureManagerGTK):
    ...

class _BackendGTK3(_BackendGTK):
    FigureCanvas: Type[FigureCanvasGTK3] = FigureCanvasGTK3
    FigureManager: Type[FigureManagerGTK3] = FigureManagerGTK3



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from ._util import arg_to_array, arg_to_vec4, as_vec4
from .base_transform import BaseTransform
from ... import gloo

class LogTransform(BaseTransform):

    # TODO: Evaluate the performance costs of using conditionals.
    # An alternative approach is to transpose the vector before
    # log-transforming, and then transpose back afterward.
    glsl_map: str = ...

    glsl_imap = ...

    Linear: bool = ...
    Orthogonal: bool = ...
    NonScaling: bool = ...
    Isometric: bool = ...

    def __init__(self, base: ArrayLike | None = None): ...
    @property
    def base(self): ...
    @base.setter
    def base(self, s): ...
    @arg_to_array
    def map(self, coords, base=None): ...
    @arg_to_array
    def imap(self, coords): ...
    def shader_map(self): ...
    def shader_imap(self): ...
    def __repr__(self): ...

class PolarTransform(BaseTransform):

    glsl_map: str = ...

    glsl_imap: str = ...

    Linear: bool = ...
    Orthogonal: bool = ...
    NonScaling: bool = ...
    Isometric: bool = ...

    @arg_to_array
    def map(self, coords): ...
    @arg_to_array
    def imap(self, coords): ...

# class BilinearTransform(BaseTransform):
#    # TODO
#    pass

# class WarpTransform(BaseTransform):
#    """ Multiple bilinear transforms in a grid arrangement.
#    """
#    # TODO

class MagnifyTransform(BaseTransform):

    glsl_map: str = ...

    glsl_imap = ...

    Linear: bool = ...

    _trans_resolution: int = ...

    def __init__(
        self,
        mag: float = 3,
        radii: tuple[float, float] = ...,
        center: tuple[float, float] = ...,
    ): ...
    @property
    def center(self): ...
    @center.setter
    def center(self, center): ...
    @property
    def mag(self): ...
    @mag.setter
    def mag(self, mag): ...
    @property
    def radii(self): ...
    @radii.setter
    def radii(self, radii): ...
    def shader_map(self): ...
    def shader_imap(self): ...
    @arg_to_vec4
    def map(self, x, _inverse=False): ...
    def imap(self, coords): ...
    def _get_transition(self): ...

class Magnify1DTransform(MagnifyTransform):

    glsl_map: str = ...

    glsl_imap = ...



from typing import Sequence
from scipy import sparse as sparse
from ..exceptions import (
    DataConversionWarning as DataConversionWarning,
    NotFittedError as NotFittedError,
    SkipTestWarning as SkipTestWarning,
)
from ..base import BaseEstimator
from ..random_projection import BaseRandomProjection as BaseRandomProjection
from inspect import signature as signature
from .validation import has_fit_parameter as has_fit_parameter
from ..metrics import (
    accuracy_score as accuracy_score,
    adjusted_rand_score as adjusted_rand_score,
    f1_score as f1_score,
)
from ..utils.fixes import sp_version as sp_version, parse_version as parse_version
from ._param_validation import Interval as Interval
from ..preprocessing import StandardScaler as StandardScaler, scale as scale
from ..linear_model import (
    LinearRegression as LinearRegression,
    LogisticRegression as LogisticRegression,
    RANSACRegressor as RANSACRegressor,
    Ridge as Ridge,
    SGDRegressor as SGDRegressor,
)
from copy import deepcopy as deepcopy
from ..datasets import (
    load_iris as load_iris,
    make_blobs as make_blobs,
    make_multilabel_classification as make_multilabel_classification,
    make_regression as make_regression,
)
from ..metrics.pairwise import (
    rbf_kernel as rbf_kernel,
    linear_kernel as linear_kernel,
    pairwise_distances as pairwise_distances,
)
from .. import config_context as config_context
from collections.abc import Generator
from ..utils.validation import check_is_fitted as check_is_fitted
from ..pipeline import make_pipeline as make_pipeline
from scipy.stats import rankdata as rankdata
from _pytest.mark.structures import MarkDecorator
from ..utils._param_validation import (
    make_constraint as make_constraint,
    generate_invalid_param_val as generate_invalid_param_val,
    InvalidParameterError as InvalidParameterError,
)
from numbers import Real as Real
from functools import partial as partial, wraps as wraps
from ..base import (
    clone as clone,
    ClusterMixin as ClusterMixin,
    is_classifier as is_classifier,
    is_regressor as is_regressor,
    is_outlier_detector as is_outlier_detector,
    RegressorMixin as RegressorMixin,
)
from ..model_selection import (
    train_test_split as train_test_split,
    ShuffleSplit as ShuffleSplit,
)
from . import IS_PYPY as IS_PYPY, is_scalar_nan as is_scalar_nan, shuffle as shuffle
from ._testing import (
    assert_raise_message as assert_raise_message,
    assert_array_equal as assert_array_equal,
    assert_array_almost_equal as assert_array_almost_equal,
    assert_allclose as assert_allclose,
    assert_allclose_dense_sparse as assert_allclose_dense_sparse,
    assert_array_less as assert_array_less,
    set_random_state as set_random_state,
    SkipTest as SkipTest,
    ignore_warnings as ignore_warnings,
    create_memmap_backed_data as create_memmap_backed_data,
    raises as raises,
)
from ..feature_selection import (
    SelectKBest as SelectKBest,
    SelectFromModel as SelectFromModel,
)
from pytest.mark import parameterize
from .._typing import ArrayLike
import types
import warnings
import pickle
import re

import numpy as np
import joblib

REGRESSION_DATASET = ...
CROSS_DECOMPOSITION: list = ...


def check_supervised_y_no_nan(name, estimator_orig):
    ...


def parametrize_with_checks(
    estimators: Sequence[BaseEstimator],
) -> MarkDecorator | parameterize:
    ...


def check_estimator(
    estimator: None | BaseEstimator = None,
    generate_only: bool = False,
    Estimator: str | BaseEstimator = "deprecated",
) -> Generator:
    ...


class _NotAnArray:
    def __init__(self, data: ArrayLike) -> None:
        ...

    def __array__(self, dtype=None):
        ...

    def __array_function__(self, func, types, args, kwargs):
        ...


def check_estimator_sparse_data(name, estimator_orig):
    ...


def check_sample_weights_pandas_series(name, estimator_orig):
    ...


def check_sample_weights_not_an_array(name, estimator_orig):
    ...


def check_sample_weights_list(name, estimator_orig):
    ...


def check_sample_weights_shape(name, estimator_orig):
    ...


def check_sample_weights_invariance(name, estimator_orig, kind: str = "ones"):
    ...


def check_sample_weights_not_overwritten(name, estimator_orig):
    ...


def check_dtype_object(name, estimator_orig):
    ...


def check_complex_data(name, estimator_orig):
    ...


def check_dict_unchanged(name, estimator_orig):
    ...


def check_dont_overwrite_parameters(name, estimator_orig):
    ...


def check_fit2d_predict1d(name, estimator_orig):
    ...


def check_methods_subset_invariance(name, estimator_orig):
    ...


def check_methods_sample_order_invariance(name, estimator_orig):
    ...


def check_fit2d_1sample(name, estimator_orig):
    ...


def check_fit2d_1feature(name, estimator_orig):
    ...


def check_fit1d(name, estimator_orig):
    ...


def check_transformer_general(name, transformer, readonly_memmap: bool = False):
    ...


def check_transformer_data_not_an_array(name, transformer):
    ...


def check_transformers_unfitted(name, transformer):
    ...


def check_pipeline_consistency(name, estimator_orig):
    ...


def check_fit_score_takes_y(name, estimator_orig):
    ...


def check_estimators_dtypes(name, estimator_orig):
    ...


def check_transformer_preserve_dtypes(name, transformer_orig):
    ...


def check_estimators_empty_data_messages(name, estimator_orig):
    ...


def check_estimators_nan_inf(name, estimator_orig):
    ...


def check_nonsquare_error(name, estimator_orig):
    ...


def check_estimators_pickle(name, estimator_orig):
    ...


def check_estimators_partial_fit_n_features(name, estimator_orig):
    ...


def check_classifier_multioutput(name, estimator):
    ...


def check_regressor_multioutput(name, estimator):
    ...


def check_clustering(name, clusterer_orig, readonly_memmap: bool = False):
    ...


def check_clusterer_compute_labels_predict(name, clusterer_orig):
    ...


def check_classifiers_one_label(name, classifier_orig):
    ...


def check_classifiers_one_label_sample_weights(name, classifier_orig):
    ...


def check_classifiers_train(
    name, classifier_orig, readonly_memmap: bool = False, X_dtype: str = "float64"
):
    ...


def check_outlier_corruption(num_outliers, expected_outliers, decision):
    ...


def check_outliers_train(name, estimator_orig, readonly_memmap: bool = True):
    ...


def check_outlier_contamination(name, estimator_orig):
    ...


def check_classifiers_multilabel_representation_invariance(name, classifier_orig):
    ...


def check_classifiers_multilabel_output_format_predict(name, classifier_orig):
    ...


def check_classifiers_multilabel_output_format_predict_proba(name, classifier_orig):
    ...


def check_classifiers_multilabel_output_format_decision_function(name, classifier_orig):
    ...


def check_get_feature_names_out_error(name, estimator_orig):
    ...


def check_estimators_fit_returns_self(
    name, estimator_orig, readonly_memmap: bool = False
):
    ...


def check_estimators_unfitted(name, estimator_orig):
    ...


def check_supervised_y_2d(name, estimator_orig):
    ...


def check_classifiers_predictions(X, y, name, classifier_orig):
    ...


def check_classifiers_classes(name, classifier_orig):
    ...


def check_regressors_int(name, regressor_orig):
    ...


def check_regressors_train(
    name, regressor_orig, readonly_memmap: bool = False, X_dtype=...
):
    ...


def check_regressors_no_decision_function(name, regressor_orig):
    ...


def check_class_weight_classifiers(name, classifier_orig):
    ...


def check_class_weight_balanced_classifiers(
    name, classifier_orig, X_train, y_train, X_test, y_test, weights
):
    ...


def check_class_weight_balanced_linear_classifier(name, Classifier):
    ...


def check_estimators_overwrite_params(name, estimator_orig):
    ...


def check_no_attributes_set_in_init(name, estimator_orig):
    ...


def check_sparsify_coefficients(name, estimator_orig):
    ...


def check_classifier_data_not_an_array(name, estimator_orig):
    ...


def check_regressor_data_not_an_array(name, estimator_orig):
    ...


def check_estimators_data_not_an_array(name, estimator_orig, X, y, obj_type):
    ...


def check_parameters_default_constructible(name, Estimator):
    ...


def check_non_transformer_estimators_n_iter(name, estimator_orig):
    ...


def check_transformer_n_iter(name, estimator_orig):
    ...


def check_get_params_invariance(name, estimator_orig):
    ...


def check_set_params(name, estimator_orig):
    ...


def check_classifiers_regression_target(name, estimator_orig):
    ...


def check_decision_proba_consistency(name, estimator_orig):
    ...


def check_outliers_fit_predict(name, estimator_orig):
    ...


def check_fit_non_negative(name, estimator_orig):
    ...


def check_fit_idempotent(name, estimator_orig):
    ...


def check_fit_check_is_fitted(name, estimator_orig):
    ...


def check_n_features_in(name, estimator_orig):
    ...


def check_requires_y_none(name, estimator_orig):
    ...


def check_n_features_in_after_fitting(name, estimator_orig):
    ...


def check_estimator_get_tags_default_keys(name, estimator_orig):
    ...


def check_dataframe_column_names_consistency(name, estimator_orig):
    ...


def check_transformer_get_feature_names_out(name, transformer_orig):
    ...


def check_transformer_get_feature_names_out_pandas(name, transformer_orig):
    ...


def check_param_validation(name, estimator_orig):
    ...


def check_set_output_transform(name, transformer_orig):
    ...


def check_set_output_transform_pandas(name, transformer_orig):
    ...


def check_global_ouptut_transform_pandas(name, transformer_orig):
    ...



from typing import Mapping
from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["is_at_free", "find_asteroidal_triple"]

def find_asteroidal_triple(G: Graph) -> list | None: ...
def is_at_free(G: Graph) -> bool: ...
def create_component_structure(G: Graph) -> Mapping: ...



from typing import Any
from ..._typing import Scalar
from ...utils.decorators import not_implemented_for, py_random_state
from ...classes.graph import Graph

__all__ = ["randomized_partitioning", "one_exchange"]

@py_random_state(1)
def randomized_partitioning(
    G: Graph, seed=None, p: Scalar = 0.5, weight: Any = None
): ...
@py_random_state(2)
def one_exchange(
    G: Graph, initial_cut: set | None = None, seed=None, weight: Any = None
): ...



from .hough_transform import (
    hough_line as hough_line,
    hough_line_peaks as hough_line_peaks,
    probabilistic_hough_line as probabilistic_hough_line,
    hough_circle as hough_circle,
    hough_circle_peaks as hough_circle_peaks,
    hough_ellipse as hough_ellipse,
)
from .radon_transform import (
    radon as radon,
    iradon as iradon,
    iradon_sart as iradon_sart,
    order_angles_golden_ratio as order_angles_golden_ratio,
)
from .finite_radon_transform import frt2 as frt2, ifrt2 as ifrt2
from .integral import integral_image as integral_image, integrate as integrate
from ._geometric import (
    estimate_transform as estimate_transform,
    matrix_transform as matrix_transform,
    EuclideanTransform as EuclideanTransform,
    SimilarityTransform as SimilarityTransform,
    AffineTransform as AffineTransform,
    ProjectiveTransform as ProjectiveTransform,
    FundamentalMatrixTransform as FundamentalMatrixTransform,
    EssentialMatrixTransform as EssentialMatrixTransform,
    PolynomialTransform as PolynomialTransform,
    PiecewiseAffineTransform as PiecewiseAffineTransform,
)
from ._warps import (
    swirl as swirl,
    resize as resize,
    rotate as rotate,
    rescale as rescale,
    downscale_local_mean as downscale_local_mean,
    warp as warp,
    warp_coords as warp_coords,
    warp_polar as warp_polar,
    resize_local_mean as resize_local_mean,
)
from .pyramids import (
    pyramid_reduce as pyramid_reduce,
    pyramid_expand as pyramid_expand,
    pyramid_gaussian as pyramid_gaussian,
    pyramid_laplacian as pyramid_laplacian,
)

__all__ = [
    "hough_circle",
    "hough_ellipse",
    "hough_line",
    "probabilistic_hough_line",
    "hough_circle_peaks",
    "hough_line_peaks",
    "radon",
    "iradon",
    "iradon_sart",
    "order_angles_golden_ratio",
    "frt2",
    "ifrt2",
    "integral_image",
    "integrate",
    "warp",
    "warp_coords",
    "warp_polar",
    "estimate_transform",
    "matrix_transform",
    "EuclideanTransform",
    "SimilarityTransform",
    "AffineTransform",
    "ProjectiveTransform",
    "EssentialMatrixTransform",
    "FundamentalMatrixTransform",
    "PolynomialTransform",
    "PiecewiseAffineTransform",
    "swirl",
    "resize",
    "resize_local_mean",
    "rotate",
    "rescale",
    "downscale_local_mean",
    "pyramid_reduce",
    "pyramid_expand",
    "pyramid_gaussian",
    "pyramid_laplacian",
]



from django.db.backends.base.features import (
    BaseDatabaseFeatures as BaseDatabaseFeatures,
)

class DummyDatabaseFeatures(BaseDatabaseFeatures):
    supports_transactions: bool = ...  # type: ignore [assignment]
    uses_savepoints: bool = ...



from numpy.typing import ArrayLike
import numpy as np
from .._shared.utils import _supported_float_type, check_nD
import itertools

def moments_coords(coords, order: int = 3): ...
def moments_coords_central(
    coords, center: tuple[float, ...] | None = None, order: int = 3
): ...
def moments(image, order: int = 3): ...
def moments_central(
    image, center: tuple[float, ...] | None = None, order: int = 3, **kwargs
): ...
def moments_normalized(mu, order: int = 3): ...
def moments_hu(nu): ...
def centroid(image: ArrayLike): ...
def inertia_tensor(image: ArrayLike, mu: ArrayLike | None = None): ...
def inertia_tensor_eigvals(image: ArrayLike, mu: ArrayLike | None = None, T=None): ...



from collections.abc import Iterable
from numpy.typing import ArrayLike
from ..classes.graph import Graph

# Original author: D. Eppstein, UC Irvine, August 12, 2003.
# The original code at http://www.ics.uci.edu/~eppstein/PADS/ is public domain.
from itertools import islice

from ..classes.graph import Graph
from ..exception import NetworkXError
from ..utils import not_implemented_for, open_file

__all__ = ["from_graph6_bytes", "read_graph6", "to_graph6_bytes", "write_graph6"]

def from_graph6_bytes(bytes_in) -> Graph: ...
def to_graph6_bytes(
    G: Graph, nodes: ArrayLike | Iterable | None = None, header: bool = True
): ...
def read_graph6(path): ...
def write_graph6(
    G: Graph, path: str, nodes: ArrayLike | Iterable | None = None, header: bool = True
): ...
def write_graph6_file(
    G: Graph, f, nodes: ArrayLike | Iterable | None = None, header: bool = True
): ...
def data_to_n(data): ...
def n_to_data(n): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse._sparsetools, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def bsr_diagonal() -> typing.Any:
    ...

def bsr_eldiv_bsr() -> typing.Any:
    ...

def bsr_elmul_bsr() -> typing.Any:
    ...

def bsr_ge_bsr() -> typing.Any:
    ...

def bsr_gt_bsr() -> typing.Any:
    ...

def bsr_le_bsr() -> typing.Any:
    ...

def bsr_lt_bsr() -> typing.Any:
    ...

def bsr_matmat() -> typing.Any:
    ...

def bsr_matvec() -> typing.Any:
    ...

def bsr_matvecs() -> typing.Any:
    ...

def bsr_maximum_bsr() -> typing.Any:
    ...

def bsr_minimum_bsr() -> typing.Any:
    ...

def bsr_minus_bsr() -> typing.Any:
    ...

def bsr_ne_bsr() -> typing.Any:
    ...

def bsr_plus_bsr() -> typing.Any:
    ...

def bsr_scale_columns() -> typing.Any:
    ...

def bsr_scale_rows() -> typing.Any:
    ...

def bsr_sort_indices() -> typing.Any:
    ...

def bsr_tocsr() -> typing.Any:
    ...

def bsr_transpose() -> typing.Any:
    ...

def coo_matvec() -> typing.Any:
    ...

def coo_tocsr() -> typing.Any:
    ...

def coo_todense() -> typing.Any:
    ...

def cs_graph_components() -> typing.Any:
    ...

def csc_diagonal() -> typing.Any:
    ...

def csc_eldiv_csc() -> typing.Any:
    ...

def csc_elmul_csc() -> typing.Any:
    ...

def csc_ge_csc() -> typing.Any:
    ...

def csc_gt_csc() -> typing.Any:
    ...

def csc_le_csc() -> typing.Any:
    ...

def csc_lt_csc() -> typing.Any:
    ...

def csc_matmat() -> typing.Any:
    ...

def csc_matmat_maxnnz() -> typing.Any:
    ...

def csc_matvec() -> typing.Any:
    ...

def csc_matvecs() -> typing.Any:
    ...

def csc_maximum_csc() -> typing.Any:
    ...

def csc_minimum_csc() -> typing.Any:
    ...

def csc_minus_csc() -> typing.Any:
    ...

def csc_ne_csc() -> typing.Any:
    ...

def csc_plus_csc() -> typing.Any:
    ...

def csc_tocsr() -> typing.Any:
    ...

def csr_column_index1() -> typing.Any:
    ...

def csr_column_index2() -> typing.Any:
    ...

def csr_count_blocks() -> typing.Any:
    ...

def csr_diagonal() -> typing.Any:
    ...

def csr_eldiv_csr() -> typing.Any:
    ...

def csr_eliminate_zeros() -> typing.Any:
    ...

def csr_elmul_csr() -> typing.Any:
    ...

def csr_ge_csr() -> typing.Any:
    ...

def csr_gt_csr() -> typing.Any:
    ...

def csr_has_canonical_format() -> typing.Any:
    ...

def csr_has_sorted_indices() -> typing.Any:
    ...

def csr_le_csr() -> typing.Any:
    ...

def csr_lt_csr() -> typing.Any:
    ...

def csr_matmat() -> typing.Any:
    ...

def csr_matmat_maxnnz() -> typing.Any:
    ...

def csr_matvec() -> typing.Any:
    ...

def csr_matvecs() -> typing.Any:
    ...

def csr_maximum_csr() -> typing.Any:
    ...

def csr_minimum_csr() -> typing.Any:
    ...

def csr_minus_csr() -> typing.Any:
    ...

def csr_ne_csr() -> typing.Any:
    ...

def csr_plus_csr() -> typing.Any:
    ...

def csr_row_index() -> typing.Any:
    ...

def csr_row_slice() -> typing.Any:
    ...

def csr_sample_offsets() -> typing.Any:
    ...

def csr_sample_values() -> typing.Any:
    ...

def csr_scale_columns() -> typing.Any:
    ...

def csr_scale_rows() -> typing.Any:
    ...

def csr_sort_indices() -> typing.Any:
    ...

def csr_sum_duplicates() -> typing.Any:
    ...

def csr_tobsr() -> typing.Any:
    ...

def csr_tocsc() -> typing.Any:
    ...

def csr_todense() -> typing.Any:
    ...

def dia_matvec() -> typing.Any:
    ...

def expandptr() -> typing.Any:
    ...

def get_csr_submatrix() -> typing.Any:
    ...

def test_throw_error() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




from skimage._typing import Scalar
from numpy import ndarray
from numpy.typing import NDArray
import numpy as np

def integral_image(image: NDArray, *, dtype=None) -> NDArray: ...
def integrate(ii: NDArray, start, end) -> Scalar | np.ndarray: ...



"""
This type stub file was generated by pyright.
"""

from traitlets.config.configurable import Configurable

"""
A context manager for handling sys.displayhook.

Authors:

* Robert Kern
* Brian Granger
"""
class DisplayTrap(Configurable):
    """Object to manage sys.displayhook.

    This came from IPython.core.kernel.display_hook, but is simplified
    (no callbacks or formatters) until more of the core is refactored.
    """
    hook = ...
    def __init__(self, hook=...) -> None:
        ...
    
    def __enter__(self): # -> Self@DisplayTrap:
        ...
    
    def __exit__(self, type, value, traceback):
        ...
    
    def set(self): # -> None:
        """Set the hook."""
        ...
    
    def unset(self): # -> None:
        """Unset the hook."""
        ...
    


